/*
 * Created on 2005-04-25
 */
package gui;

import gui.controls.LocationsChooser;
import gui.dialogs.MainDialog;
import java.util.ArrayList;
import scheduler.TestOptions;
import scheduler.TravelConstraints;
import scheduler.TravelPartsGenerator;
import scheduler.modules.FeedbackTravelPartsProviderStub;
import scheduler.modules.MeansOfTransportationChooserStub;
import scheduler.modules.MeansOfTransportationJessChooser;
import scheduler.modulesinterfaces.MeansOfTransportationProvider;
import world.WorldTravelInfoProvider;

/**
 * A thread used to solve the first step of the scheduling algorithm.
 * @author Piotrek
 */
public final class FirstStepSolverThread extends Thread
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param mainDialog The main dialog to be notified of the results.
	 * @param travelConstraints The travel constraints.
	 */
	public FirstStepSolverThread(MainDialog mainDialog, TravelConstraints travelConstraints)
	{
		if (mainDialog == null)
			throw new NullPointerException("Main dialog must not be null in FirstStepSolverThread.FirstStepSolverThread().");
		
		if (travelConstraints == null)
			throw new NullPointerException("Travel constraints must not be null in FirstStepSolverThread.FirstStepSolverThread().");
		
		this.mainDialog = mainDialog;
		this.travelConstraints = travelConstraints;
	}
	
	/**
	 * The main dialog to be notified of the results.
	 * @author Piotrek
	 */
	private MainDialog mainDialog = null;
	
	/**
	 * The travel constraints.
	 * @author Piotrek
	 */
	private TravelConstraints travelConstraints = null;
	
	/**
	 * @author Piotrek
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		WorldTravelInfoProvider worldTravelInfoProvider = new WorldTravelInfoProvider();
		
		TravelPartsGenerator travelPartsGenerator = new TravelPartsGenerator();
		
		MeansOfTransportationProvider meansOfTransportationChooser = null;
		
		switch (TestOptions.GetMeansOfTransportationChoosingEngine())
		{
			case TestOptions.MEANS_OF_TRANSPORTATION_CHOOSING_RULES_ENGINE_JAVA:
					
				meansOfTransportationChooser = new MeansOfTransportationChooserStub();
			
				break;
				
			case TestOptions.MEANS_OF_TRANSPORTATION_CHOOSING_RULES_ENGINE_JESS:
				
				meansOfTransportationChooser = new MeansOfTransportationJessChooser();
			
				break;
				
			default:
				
				throw new RuntimeException("Unexpected means of transportation choosing engine.");
		}
						
		FeedbackTravelPartsProviderStub feedbackTravelPartsProvider = new FeedbackTravelPartsProviderStub();
		
		feedbackTravelPartsProvider.SetTravelConstraints(travelConstraints);
		
		travelPartsGenerator.SetTravelInfoProvider(worldTravelInfoProvider);
		travelPartsGenerator.SetMeansOfTransportationProvider(meansOfTransportationChooser);
		travelPartsGenerator.SetFeedbackTravelPartsProvider(feedbackTravelPartsProvider);
		
		LocationsChooser locationsChooser = mainDialog.GetLocationsChooser();
		
		TestOptions.ResetCounters();
						
		try
		{
			ArrayList solutions = travelPartsGenerator.GenerateTravelParts(locationsChooser.GetOrigin(), locationsChooser.GetDestination(), travelConstraints);
		
			mainDialog.OnFirstStepReady(solutions);
		}
		
		catch (OutOfMemoryError xcp)
		{
			mainDialog.OnSchedulingAlgorithmStepError("Out of memory");
		}
		
		catch (Throwable t)
		{
			mainDialog.OnSchedulingAlgorithmStepError(t.getMessage());
		}
	}
}
