/*
 * Created on 2005-04-01
 */
package gui.controls;

import java.awt.event.ActionListener;
import world.Airport;
import world.Location;

/**
 * A panel for edition of properties of an airport.
 * @author Piotrek
 */
public class AirportPropertiesPanel extends BasicPropertiesPanel
{
	/**
	 * Constructor.
	 * @param actionListener An action listener for the buttons.
	 * @author Piotrek
	 */
	public AirportPropertiesPanel(ActionListener actionListener)
	{
		// the base class checks for null value
		super(actionListener, "Airport Properties");
	}
	
	/**
	 * Stores currently set airport properties to the given airport.
	 * @author Piotrek
	 * @param airport An airport to receive the currrently set properties.
	 */
	public void StoreData(Airport airport)
	{
		if (airport == null)
			throw new NullPointerException("Airport must not be null in AirportPropertiesPanel.StoreData().");
		
		if (nameTextField.getText().compareTo("") == 0)
			airport.name = "Anonymous Airport";
		else
			airport.name = nameTextField.getText();
		
		Location location = airport.GetLocation();
		
		location.continentName = (String)continentNameComboBox.getSelectedItem();
		location.islandName = (String)islandNameComboBox.getSelectedItem();
	}
	
	/**
	 * Loads the properties of the given airport.
	 * @author Piotrek
	 * @param airport A source airport.
	 */
	public void LoadData(Airport airport)
	{
		if (airport == null)
			throw new NullPointerException("Airport must not be null in AirportPropertiesPanel.LoadData().");
		
		nameTextField.setText(airport.name);
		
		Location location = airport.GetLocation();
		
		continentNameComboBox.setSelectedItem(location.continentName);
		islandNameComboBox.setSelectedItem(location.islandName);
	}
}