/*
 * Created on 2005-04-03
 */
package gui.controls;

import java.awt.event.ActionListener;
import world.BusStop;
import world.Location;

/**
 * A panel for edition of properties of a bus stop.
 * @author Piotrek
 */
public class BusStopPropertiesPanel extends BasicPropertiesPanel
{
	/**
	 * Constructor.
	 * @param actionListener An action listener for the buttons.
	 * @author Piotrek
	 */
	public BusStopPropertiesPanel(ActionListener actionListener)
	{
		// base class checks for null value
		super(actionListener, "Bus Stop Properties");
	}
	
	/**
	 * Stores currently set bus stop properties to the given bus stop.
	 * @author Piotrek
	 * @param busStop A bus stop to receive the currrently set properties.
	 */
	public void StoreData(BusStop busStop)
	{
		if (busStop == null)
			throw new NullPointerException("Bus stop must not be null in BusStopPropertiesPanel.StoreData().");
		
		if (nameTextField.getText().compareTo("") == 0)
			busStop.name = "Anonymous Bus Stop";
		else
			busStop.name = nameTextField.getText();
		
		Location location = busStop.GetLocation();
		
		location.continentName = (String)continentNameComboBox.getSelectedItem();
		location.islandName = (String)islandNameComboBox.getSelectedItem();
	}
	
	/**
	 * Loads the properties of the given bus stop.
	 * @author Piotrek
	 * @param busStop A source bus stop.
	 */
	public void LoadData(BusStop busStop)
	{
		if (busStop == null)
			throw new NullPointerException("Bus stop must not be null in BusStopPropertiesPanel.LoadData().");
		
		nameTextField.setText(busStop.name);
		
		Location location = busStop.GetLocation();
		
		continentNameComboBox.setSelectedItem(location.continentName);
		islandNameComboBox.setSelectedItem(location.islandName);
	}
}
