/*
 * Created on 2005-03-30
 */
package gui.controls;

import java.awt.event.ActionListener;
import world.City;
import world.Location;

/**
 * A panel containging a set of properties of a city.
 * @author Piotrek
 */
public class CityPropertiesPanel extends BasicPropertiesPanel
{
	/**
	 * Constructor.
	 * @param actionListener An action listener for the buttons.
	 * @author Piotrek
	 */
	public CityPropertiesPanel(ActionListener actionListener)
	{
		// the base class checks for null value
		super(actionListener, "City Properties");
	}
	
	/**
	 * Stores currently set city properties to the given city.
	 * @author Piotrek
	 * @param city A city to receive the currrently set properties.
	 */
	public void StoreData(City city)
	{
		if (city == null)
			throw new NullPointerException("City must not be null in CityPropertiesPanel.StoreData().");
		
		if (nameTextField.getText().compareTo("") == 0)
			city.name = "Anonymous City";
		else
			city.name = nameTextField.getText();
		
		Location location = city.GetLocation();
		
		location.continentName = (String)continentNameComboBox.getSelectedItem();
		location.islandName = (String)islandNameComboBox.getSelectedItem();
	}
	
	/**
	 * Loads the properties of the given city.
	 * @author Piotrek
	 * @param city A source city.
	 */
	public void LoadData(City city)
	{
		if (city == null)
			throw new NullPointerException("City must not be null in CityPropertiesPanel.LoadData().");
		
		nameTextField.setText(city.name);
		
		Location location = city.GetLocation();
		
		continentNameComboBox.setSelectedItem(location.continentName);
		islandNameComboBox.setSelectedItem(location.islandName);
	}
}
