/*
 * Created on 2005-04-06
 */
package gui.controls;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import world.Location;
import world.Station;

/**
 * A cell renderer for the stations.
 * @author Piotrek
 */
public class StationsCellRenderer extends JPanel implements ListCellRenderer
{
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public StationsCellRenderer()
	{
		super();
				
		setLayout(null);
	}
	
	/**
	 * @author Piotrek
	 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
	 */
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
	{
		station = (Station)value;
		
		itemIndex = index + 1;
	       
		if (isSelected)
		{
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
		}
        else
        {
        	setBackground(list.getBackground());
        	setForeground(list.getForeground());
        }
		
		setEnabled(list.isEnabled());
		setFont(list.getFont());
	                
        return this;
	}
	
	/**
	 * A station we are rendering.
	 * @author Piotrek
	 */
	private Station station = null;
	
	/**
	 * A one-based index on the list of the station we are rendering.
	 * @author Piotrek
	 */
	private int itemIndex = -1;
	
	/**
	 * @author Piotrek
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		super.paint(g);
		
		paintChildren(g);
		
		Location location = station.GetLocation();
		
		g.drawString("Number: " + itemIndex, 10, 15);
		g.setColor(Color.BLUE);
		g.drawString("Name: " + station.name, 10, 30);
		g.setColor(Color.BLACK);
		g.drawString("Continent: " + location.continentName, 10, 45);		
		g.drawString("Island: " + location.islandName, 10, 60);
		
		if (itemIndex != 1)
			g.drawLine(0, 0, getWidth(), 0);
	}
}
