/*
 * Created on 2005-04-03
 */
package gui.controls;

import java.awt.event.ActionListener;
import world.Location;
import world.TrainStation;

/**
 * A panel for edition of properties of a train station.
 * @author Piotrek
 */
public class TrainStationPropertiesPanel extends BasicPropertiesPanel
{
	/**
	 * Constructor.
	 * @param actionListener An action listener for the buttons.
	 * @author Piotrek
	 */
	public TrainStationPropertiesPanel(ActionListener actionListener)
	{
		// the base class checks for null values
		super(actionListener, "Train Station Properties");
	}	
	
	/**
	 * Stores currently set train station properties to the given train station.
	 * @author Piotrek
	 * @param trainStation A train station to receive the currrently set properties.
	 */
	public void StoreData(TrainStation trainStation)
	{
		if (trainStation == null)
			throw new NullPointerException("Train station must not be null in TrainStationPropertiesPanel.StoreData().");
		
		if (nameTextField.getText().compareTo("") == 0)
			trainStation.name = "Anonymous Train Station";
		else
			trainStation.name = nameTextField.getText();
		
		Location location = trainStation.GetLocation();
		
		location.continentName = (String)continentNameComboBox.getSelectedItem();
		location.islandName = (String)islandNameComboBox.getSelectedItem();
	}
	
	/**
	 * Loads the properties of the given train station.
	 * @author Piotrek
	 * @param trainStation A source train station.
	 */
	public void LoadData(TrainStation trainStation)
	{
		if (trainStation == null)
			throw new NullPointerException("Train station must not be null in TrainStationPropertiesPanel.LoadData().");
		
		nameTextField.setText(trainStation.name);
		
		Location location = trainStation.GetLocation();
		
		continentNameComboBox.setSelectedItem(location.continentName);
		islandNameComboBox.setSelectedItem(location.islandName);
	}
}
