/*
 * Created on 2005-05-03
 */
package gui.controls;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import scheduler.TravelPartsList;

/**
 * A list used to choose one of the generated travel divisions to be displayed in the view.
 * @author Piotrek
 */
public final class TravelDivisionsChooser extends JPanel implements ListSelectionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param worldView A view showing the selected travel division.
	 * @param actionListener An action listener for the `Launch second step' button.
	 */
	public TravelDivisionsChooser(WorldView worldView, ActionListener actionListener)
	{
		super(null);
		
		if (worldView == null)
			throw new NullPointerException("World view must not be null in TravelDivisionsChooser.TravelDivisionsChooser().");
		
		if (actionListener == null)
			throw new NullPointerException("Action listener must not be null in TravelDivisionsChooser.TravelDivisionsChooser().");
		
		this.worldView = worldView;
		
		worldView.SetTravelDivisionChooser(this);
		
		setBackground(Color.BLUE);
		
		// creating a list view
		 
		divisionsList = new JList();
		
		divisionsList.setModel(listModel);
		divisionsList.setCellRenderer(new TravelDivisionsCellRenderer());
		
		divisionsList.addListSelectionListener(this);
	
		divisionsList.setFixedCellWidth(150);
		divisionsList.setFixedCellHeight(50);
				
		listPane = new JScrollPane(divisionsList);
		
		add(listPane);
		
		divisionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		// second step button
		launchSecondStepButton = new JButton("Launch the second step");
		
		add(launchSecondStepButton);
		
		launchSecondStepButton.setBackground(Color.ORANGE);
		launchSecondStepButton.setEnabled(false);
		
		launchSecondStepButton.addActionListener(actionListener);
						
		setVisible(false);
	}
	
	/**
	 * A button used to launch the second step of the scheduling algorithm.
	 * @author Piotrek
	 */
	private JButton launchSecondStepButton = null;
	
	/**
	 * Gets the `launch the second step' button.
	 * @author Piotrek
	 * @return A `launch the second step' button.
	 */
	public JButton GetLaunchSecondStepButton()
	{
		return launchSecondStepButton;
	}
	
	/**
	 * A view displaying the selected travel division.
	 * @author Piotrek
	 */
	private WorldView worldView = null;
	
	/**
	 * A list control displaying the travel divisions.
	 * @author Piotrek
	 */
	private JList divisionsList = null;
	
	/**
	 * The list model providing an API to add items to the list.
	 * @author Piotrek
	 */
	private DefaultListModel listModel = new DefaultListModel();
		
	/**
	 * The scroll pane making the list scrollable.
	 * @author Piotrek
	 */
	private JScrollPane listPane;
	
	/**
	 * @author Piotrek
	 * @see JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g)
	{
		super.paint(g);
		
		g.setColor(getBackground());
		g.fillRect(0, 0 , getSize().width, getSize().height);
						
		paintChildren(g);
	}
	
	/**
	 * @author Piotrek
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x, y, width, height);
		
		listPane.setBounds(5, 5, width - 10, height - 50);
		
		launchSecondStepButton.setBounds(5, height - 35, width - 10, 25);
	}

	/**
	 * @author Piotrek
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		worldView.repaint();
	}
	
	/**
	 * Loads the given list of travel divisions into the list control.
	 * @author Piotrek
	 * @param divisions A list of travel divisions to be loaded.
	 */
	public void LoadDivisions(ArrayList divisions)
	{
		if (divisions == null)
			throw new NullPointerException("Divisions must not be null in TravelDivisionsChooser.LoadDivisions().");
		
		listModel.clear();
		
		Iterator iterator = divisions.iterator();
		
		while (iterator.hasNext())
		{
			TravelPartsList division = (TravelPartsList)iterator.next();
			
			listModel.addElement(division);
		}
		
		launchSecondStepButton.setEnabled(divisions.size() > 0);
	}
	
	/**
	 * Gets the currently selected division.
	 * @author Piotrek
	 * @return A currently selected division or a null value if there is none.
	 */
	public TravelPartsList GetSelectedDivision()
	{
		TravelPartsList selectedDivision = (TravelPartsList)divisionsList.getSelectedValue();
		
		return selectedDivision;
	}
}
