/*
 * Created on 2005-06-16
 */
package gui.dialogs;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import scheduler.TravelSolutionsSorter;

/**
 * A dialog used to choose a property to sort the travel solutions on.
 * @author Piotrek
 */
public final class SortPropertyChooser extends JDialog implements ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 */
	public SortPropertyChooser()
	{
		super();
				
		setModal(true);
		setTitle("Question");
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(405, 400);
		setResizable(false);
		
		// panel	
		panel = getContentPane();
		
		panel.setLayout(null);
		
		setContentPane(panel);
		
		// adding a message
		
		JLabel label = new JLabel("Choose the property to sort the travel solutions on", SwingConstants.CENTER);
		
		add(label);
		
		label.setBounds(10, 25, 380, 25);
		label.setBackground(Color.YELLOW);
		label.setOpaque(true);
		
		// adding radio buttons
		
		// sort on average change duration
		
		averageChangeDurationOption = new JRadioButton("Average change duration", true);
		
		averageChangeDurationOption.setBounds(110, 80, 180, 25);
		averageChangeDurationOption.addActionListener(this);
		
		add(averageChangeDurationOption);
		
		// max change duration
		
		maxChangeDurationOption = new JRadioButton("Max change duration", false);
		
		maxChangeDurationOption.setBounds(110, 115, 180, 25);
		maxChangeDurationOption.addActionListener(this);
	
		add(maxChangeDurationOption);
		
		// changes count
		
		changesCountOption = new JRadioButton("Changes count", false);
		
		changesCountOption.setBounds(110, 150, 180, 25);
		changesCountOption.addActionListener(this);
		
		add(changesCountOption);
		
		// similarities to the history
		
		historySimilarityOption = new JRadioButton("Similarity to the history", false);
		
		historySimilarityOption.setBounds(110, 185, 180, 25);
		historySimilarityOption.addActionListener(this);
		
		add(historySimilarityOption);
		
		// total distance
		totalDistanceOption = new JRadioButton("Total distance", false);
		
		totalDistanceOption.setBounds(110, 220, 180, 25);
		totalDistanceOption.addActionListener(this);
		
		add(totalDistanceOption);
		
		// total duration
		totalDurationOption = new JRadioButton("Total duration", false);
		
		totalDurationOption.setBounds(110, 255, 180, 25);
		totalDurationOption.addActionListener(this);
		
		add(totalDurationOption);		
		
		// grouping radio buttons
		
		ButtonGroup buttonGroup = new ButtonGroup();
		
		buttonGroup.add(averageChangeDurationOption);
		buttonGroup.add(maxChangeDurationOption);
		buttonGroup.add(changesCountOption);
		buttonGroup.add(historySimilarityOption);
		buttonGroup.add(totalDistanceOption);
		buttonGroup.add(totalDurationOption);
		
		// ok and cancel buttons
		
		// ok
		
		okButton = new JButton("OK");
		
		okButton.setBounds(110, 310, 80, 25);
		okButton.setBackground(Color.ORANGE);
		
		okButton.addActionListener(this);
		
		add(okButton);
		
		// cancel
		
		cancelButton = new JButton("Cancel");
		
		cancelButton.setBounds(210, 310, 80, 25);
		cancelButton.setBackground(Color.ORANGE);
		
		cancelButton.addActionListener(this);
		
		add(cancelButton);
	}
	
	/**
	 * A main panel of this dialog.
	 * @author Piotrek
	 */
	private Container panel = null;
	
	/**
	 * An OK button.
	 * @author Piotrek
	 */
	JButton okButton = null;
	
	/**
	 * A cancel button.
	 * @author Piotrek
	 */
	JButton cancelButton = null;
	
	/**
	 * The OK dialog result.
	 * @author Piotrek
	 */
	public static final int OK = 1;
	
	/**
	 * The cancel dialog result.
	 * @author Piotrek
	 */
	public static final int CANCEL = 2;
	
	/**
	 * The dialog result.
	 * @author Piotrek
	 */
	private int dialogResult = CANCEL;
	
	/**
	 * Gets the dialog result.
	 * @author Piotrek
	 * @return The dialog result.
	 */
	public int GetDialogResult()
	{
		return dialogResult;
	}
	
	/**
	 * A radio button with the average change duration.
	 * @author Piotrek
	 */
	JRadioButton averageChangeDurationOption = null;
	
	/**
	 * A radio button with the max change duration.
	 * @author Piotrek
	 */
	JRadioButton maxChangeDurationOption = null;
	
	/**
	 * A radio button with the changes count.
	 * @author Piotrek
	 */
	JRadioButton changesCountOption = null;
	
	/**
	 * A radio button with the history similarity.
	 * @author Piotrek
	 */
	JRadioButton historySimilarityOption = null;
	
	/**
	 * A radio button with the total distance.
	 * @author Piotrek
	 */
	JRadioButton totalDistanceOption = null;
	
	/**
	 * A radio button with the total duration.
	 * @author Piotrek
	 */
	JRadioButton totalDurationOption = null;
	
	/**
	 * A mean of transportation chosen by the user.
	 * @author Piotrek
	 */
	private int chosenSortProperty = TravelSolutionsSorter.SORT_ON_AVERAGE_CHANGE_DURATION;
	
	/**
	 * Gets a sort property chosen by the user.
	 * @author Piotrek
	 * @return A chosen sort property.
	 * @see TravelSolutionsSorter#SORT_ON_AVERAGE_CHANGE_DURATION
	 * @see TravelSolutionsSorter#SORT_ON_CHANGES_COUNT
	 * @see TravelSolutionsSorter#SORT_ON_MAX_CHANGE_DURATION
	 * @see TravelSolutionsSorter#SORT_ON_SIMILARITIES_TO_HISTORY
	 * @see TravelSolutionsSorter#SORT_ON_TOTAL_DISTANCE
	 * @see TravelSolutionsSorter#SORT_ON_TOTAL_DURATION
	 */
	public int GetChosenSortProperty()
	{
		return chosenSortProperty;
	}
	
	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 * @author Piotrek
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okButton)
		{
			setVisible(false);
			
			dialogResult = OK;
		}
		
		if (e.getSource() == cancelButton)
		{
			setVisible(false);
			
			dialogResult = CANCEL;
		}
		
		if (e.getSource() == averageChangeDurationOption)
			chosenSortProperty = TravelSolutionsSorter.SORT_ON_AVERAGE_CHANGE_DURATION;
		
		if (e.getSource() == maxChangeDurationOption)
			chosenSortProperty = TravelSolutionsSorter.SORT_ON_MAX_CHANGE_DURATION;
		
		if (e.getSource() == changesCountOption)
			chosenSortProperty = TravelSolutionsSorter.SORT_ON_CHANGES_COUNT;
			
		if (e.getSource() == historySimilarityOption)
			chosenSortProperty = TravelSolutionsSorter.SORT_ON_SIMILARITIES_TO_HISTORY;
		
		if (e.getSource() == totalDistanceOption)
			chosenSortProperty = TravelSolutionsSorter.SORT_ON_TOTAL_DISTANCE;
		
		if (e.getSource() == totalDurationOption)
			chosenSortProperty = TravelSolutionsSorter.SORT_ON_TOTAL_DURATION;
	}
}
