/*
 * Created on 2005-04-10
 */
package gui.dialogs;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import world.TravelTime;

/**
 * A dialog that shows all the times of a single day for a connection with the given 
 * start time and repetition time span.
 * @author Piotrek
 */
public final class TimesPerDayPreviewDialog extends JDialog implements ActionListener
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param startTime A time one of the runs for the connection starts at (in minutes since midnight).
	 * @param repetitionTimeSpan A time between runs for the connection (in minutes).
	 * @param lowerLimit A lower limit of repetitions (in minutes since midnight).
	 * @param upperLimit An upper limit of repetitions (in minutes since midnight).
	 */
	public TimesPerDayPreviewDialog(int startTime, int repetitionTimeSpan, int lowerLimit, int upperLimit) 
	{
		if (startTime < 0 || startTime > 1439)
			throw new IllegalArgumentException("The start time in TimesPerDayPreviewDialog.TimesPerDayPreviewDialog() is invalid.");
		
		if (repetitionTimeSpan < 1 || repetitionTimeSpan > 1439)
			throw new IllegalArgumentException("The repetition time span in TimesPerDayPreviewDialog.TimesPerDayPreviewDialog() is invalid.");
		
		if (lowerLimit < 0 || lowerLimit > 1439)
			throw new IllegalArgumentException("The lower time limit in TimesPerDayPreviewDialog.TimesPerDayPreviewDialog() is invalid.");
		
		if (upperLimit < 0 || upperLimit > 1439)
			throw new IllegalArgumentException("The upper time limit in TimesPerDayPreviewDialog.TimesPerDayPreviewDialog() is invalid.");
		
		if (lowerLimit >= upperLimit)
			throw new IllegalArgumentException("The lower time limit must be less then the upper time limit in TimesPerDayPreviewDialog.TimesPerDayPreviewDialog() is invalid.");
				
		this.startTime = startTime;
		this.repetitionTimeSpan = repetitionTimeSpan;
		this.lowerLimit = lowerLimit;
		this.upperLimit = upperLimit;
		
		setSize(215, 500);
		setBackground(Color.BLUE);
		setResizable(false);
		setModal(true);
		setLayout(null);
		setTitle("Preview");
		
		// label
		JLabel label = new JLabel("Times of runs:", SwingConstants.CENTER);
		
		label.setBackground(Color.YELLOW);
		label.setBounds(10, 10, 180, 20);
		label.setOpaque(true);
		
		add(label);
		
		// text area
		textArea = new JTextArea();
		
		textArea.setAutoscrolls(true);
		textArea.setEditable(false);
		
		JScrollPane scroller = new JScrollPane(textArea);
		
		scroller.setBounds(10, 50, 180, 350);
		
		add(scroller);
		
		// OK button
		okButton = new JButton("OK");
		
		okButton.setBackground(Color.ORANGE);
		okButton.setBounds(50, 420, 100, 25);
		okButton.addActionListener(this);
		
		add(okButton);
		
		// loading the times of runs
		LoadTimes();
	}
	
	/**
	 * An OK button.
	 * @author Piotrek
	 */
	JButton okButton = null;
	
	/**
	 * A text area used to show the times of runs.
	 * @author Piotrek
	 */
	private JTextArea textArea = null;
	
	/**
	 * A time one of the runs for the connection starts at (in minutes since midnight).
	 * @author Piotrek
	 */
	private int startTime = 0;
	
	/**
	 * A time between runs for the connection (in minutes).
	 * @author Piotrek
	 */
	private int repetitionTimeSpan = 0;
	
	/**
	 * A lower limit of repetitions (in minutes since midnight).
	 * @author Piotrek
	 */
	private int lowerLimit = 0;
	
	/**
	 * An upper limit of repetitions (in minutes since midnight).
	 * @author Piotrek
	 */
	private int upperLimit = 0;
	
	/**
	 * @author Piotrek
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okButton)
		{
			setVisible(false);
		}
	}
	
	/**
	 * Loads the times of runs into the text area.
	 * @author Piotrek
	 */
	private void LoadTimes()
	{
		ArrayList times = TravelTime.GetTimesPerDay(startTime, repetitionTimeSpan, lowerLimit, upperLimit);
		
		Iterator iterator = times.iterator();
		
		while (iterator.hasNext())
		{
			Integer timeInteger = (Integer)iterator.next();
			
			textArea.append(TravelTime.GetTime(timeInteger.intValue()) + "\n");
		}
		
		if (times.size() == 0)
			textArea.append("You have defined an empty set.");
	}
}
