/*
 * Created on 2005-04-30
 */
package scheduler;

import world.Location;
import world.MeanOfTransportation;
import world.Station;

/**
 * A travel part.
 * @author Piotrek
 */
public class TravelPart
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param meanOfTransportation The mean of transportation this travel part is to be taken by.
	 * @param origin The origin location of this travel part.
	 * @param destination The destination location of this travel part.
	 * @see world.MeanOfTransportation#AIRPLANE
	 * @see world.MeanOfTransportation#BUS
	 * @see world.MeanOfTransportation#TRAIN
	 * @see world.MeanOfTransportation#SHIP
	 */
	public TravelPart(int meanOfTransportation, Location origin, Location destination)
	{
		if (!MeanOfTransportation.IsValidMeanOfTransportation(meanOfTransportation))
			throw new IllegalArgumentException("The mean of transportation must be a valid value in TravelPart.TravelPart().");
		
		this.meanOfTransportation = meanOfTransportation;
		
		if (origin == null)
			throw new NullPointerException("The origin of the travel part must not be null in TravelPart.TravelPart().");
		
		if (destination == null)
			throw new NullPointerException("The destination of the travel part must not be null in TravelPart.TravelPart().");
		
		this.origin = origin;
		this.destination = destination;
	}
	
	/**
	 * A mean of transportation this part is to be taken by.
	 * @author Piotrek
	 * @see world.MeanOfTransportation#AIRPLANE
	 * @see world.MeanOfTransportation#BUS
	 * @see world.MeanOfTransportation#TRAIN
	 * @see world.MeanOfTransportation#SHIP
	 */
	private int meanOfTransportation = -1;
	
	/**
	 * Gets the mean of transportations this travel part is to be taken by.
	 * @author Piotrek
	 * @see world.MeanOfTransportation#AIRPLANE
	 * @see world.MeanOfTransportation#BUS
	 * @see world.MeanOfTransportation#TRAIN
	 * @see world.MeanOfTransportation#SHIP
	 * @return Returns the mean of transportation this travel part is to be taken by.
	 */
	public int GetMeanOfTransportation()
	{
		return meanOfTransportation;
	}
	
	/**
	 * The origin of this travel part.
	 * @author Piotrek
	 */
	private Location origin = null;
	
	/**
	 * Gets the origin of this travel part.
	 * @author Piotrek
	 * @return The location of this tracel part.
	 */
	public Location GetOrigin()
	{
		return origin;
	}
	
	/**
	 * The destination of this travel part.
	 * @author Piotrek
	 */
	private Location destination = null;
	
	/**
	 * Gets ths destination of this travel part.
	 * @author Piotrek
	 * @return The destination of this travel part.
	 */
	public Location GetDestination()
	{
		return destination;
	}
	
	/**
	 * A station related to the origin.
	 * @author Piotrek
	 */
	private Station originStation = null;
	
	/**
	 * Sets the station related to the origin.
	 * @author Piotrek
	 * @param station A station to be set.
	 */
	public void SetOriginStation(Station station)
	{
		originStation = station;
	}
	
	/**
	 * Gets the origin related station.
	 * @author Piotrek
	 * @return The origin related station or null if there is none set.
	 */
	public Station GetOriginStation()
	{
		return originStation;
	}
	
	/**
	 * A station related to the destination.
	 * @author Piotrek
	 */
	private Station destinationStation = null;
	
	/**
	 * Sets the station related to the destination.
	 * @author Piotrek
	 * @param station A station to be set.
	 */
	public void SetDestinationStation(Station station)
	{
		destinationStation = station;
	}
	
	/**
	 * Gets the destination related station.
	 * @author Piotrek
	 * @return The destination related station or null if there is none set.
	 */
	public Station GetDestinationStation()
	{
		return destinationStation;
	}
	
	/**
	 * Gets the reverted travel part.  
	 * @author Piotrek
	 * @return The reverted travel part.
	 */
	public TravelPart GetRevertedPart()
	{
		TravelPart revertedTravelPart = new TravelPart(meanOfTransportation, destination, origin);
		
		revertedTravelPart.SetOriginStation(destinationStation);
		revertedTravelPart.SetDestinationStation(originStation);
		
		return revertedTravelPart;
	}
}
