/*
 * Created on 2005-08-29
 */
package scheduler.modules;

import java.util.ArrayList;
import scheduler.TestOptions;
import scheduler.modulesinterfaces.MeansOfTransportationProvider;
import world.Location;
import world.MeanOfTransportation;

/**
 * A stub providing a set of rules to choose a proper means of transportation for the two given locations.
 * @author Piotrek
 */
public final class MeansOfTransportationChooserStub implements MeansOfTransportationProvider
{
	/**
	 * @author Piotrek
	 * @see scheduler.modulesinterfaces.MeansOfTransportationProvider#GetMeansOfTransportation(world.Location, world.Location)
	 */
	public ArrayList GetMeansOfTransportation(Location origin, Location destination)
	{
		if (origin == null)
			throw new NullPointerException("Origin must not be null in MeansOfTransportationChooserStub.GetMeansOfTransportation().");
		
		if (destination == null)
			throw new NullPointerException("Destination must not be null in MeansOfTransportationChooserStub.GetMeansOfTransportation().");
		
		ArrayList meansOfTransportation = new ArrayList();
		
		double distance = Location.GetDistance(origin, destination);
				
		if (TestOptions.ShouldUseRulesChoosingMeansOfTransportation()) // TIP: this 'if' and 'else' might be removed in production version
		{
			// airplane rules
			if (distance >= MIN_FLIGHT_DISTANCE)
				meansOfTransportation.add(new Integer(MeanOfTransportation.AIRPLANE));
			else
			{
				if (!TestOptions.ShouldIgnoreContinentIslandInformation()) // TIP: this 'if' might be removed in production version
				{
					if (origin.continentName.compareTo("") != 0 && destination.continentName.compareTo("") != 0)
					{
						if (origin.continentName.compareTo(destination.continentName) != 0)
							meansOfTransportation.add(new Integer(MeanOfTransportation.AIRPLANE));
						else
						{
							if (origin.islandName.compareTo("") != 0 && destination.islandName.compareTo("") != 0)
								if (origin.islandName.compareTo(destination.islandName) != 0)
									meansOfTransportation.add(new Integer(MeanOfTransportation.AIRPLANE));
						}
					}
				}
			}
			
			// train rules
			if (distance >= MIN_TRAIN_DISTANCE)
			{
				if (!TestOptions.ShouldIgnoreContinentIslandInformation()) // TIP: this 'if' might be removed in production version
				{
					if (origin.continentName.compareTo("") != 0 && destination.continentName.compareTo("") != 0)
					{
						if (origin.continentName.compareTo(destination.continentName) == 0)
						{
							if (origin.islandName.compareTo("") != 0 && destination.islandName.compareTo("") != 0)
							{
								if (origin.islandName.compareTo(destination.islandName) == 0)
									meansOfTransportation.add(new Integer(MeanOfTransportation.TRAIN));
							}
							else
								meansOfTransportation.add(new Integer(MeanOfTransportation.TRAIN));
						}
					}
					else
						meansOfTransportation.add(new Integer(MeanOfTransportation.TRAIN));
				}
				else
					meansOfTransportation.add(new Integer(MeanOfTransportation.TRAIN));
			}
			
			// bus rules
			if (distance >= MIN_BUS_DISTANCE)
			{
				if (!TestOptions.ShouldIgnoreContinentIslandInformation()) // TIP: this 'if' might be removed in production version
				{
					if (origin.continentName.compareTo("") != 0 && destination.continentName.compareTo("") != 0)
					{
						if (origin.continentName.compareTo(destination.continentName) == 0)
						{
							if (origin.islandName.compareTo("") != 0 && destination.islandName.compareTo("") != 0)
							{
								if (origin.islandName.compareTo(destination.islandName) == 0)
									meansOfTransportation.add(new Integer(MeanOfTransportation.BUS));
							}
							else
								meansOfTransportation.add(new Integer(MeanOfTransportation.BUS));
						}
					}
					else
						meansOfTransportation.add(new Integer(MeanOfTransportation.BUS));
				}
				else
					meansOfTransportation.add(new Integer(MeanOfTransportation.BUS));
			}
			
			// ship rules
			if (distance >= MIN_CRUISE_DISTANCE)
				meansOfTransportation.add(new Integer(MeanOfTransportation.SHIP));
				
			// walk rules
			if (distance <= MAX_WALK_DISTANCE)
				meansOfTransportation.add(new Integer(MeanOfTransportation.WALK));
				
			// taxi rules		
			if (distance >= MIN_TAXI_DISTANCE && distance <= MAX_TAXI_DISTANCE && distance > MAX_WALK_DISTANCE)
			{
				if (!TestOptions.ShouldIgnoreContinentIslandInformation()) // TIP: this 'if' might be removed in production version
				{
					if (origin.continentName.compareTo("") != 0 && destination.continentName.compareTo("") != 0)
					{
						if (origin.continentName.compareTo(destination.continentName) == 0)
						{
							if (origin.islandName.compareTo("") != 0 && destination.islandName.compareTo("") != 0)
							{
								if (origin.islandName.compareTo(destination.islandName) == 0)
									meansOfTransportation.add(new Integer(MeanOfTransportation.TAXI));
							}
							else
								meansOfTransportation.add(new Integer(MeanOfTransportation.TAXI));
						}
					}
					else
						meansOfTransportation.add(new Integer(MeanOfTransportation.TAXI));
				}
				else
					meansOfTransportation.add(new Integer(MeanOfTransportation.TAXI));
			}
		}
		else
		{
			// walk simple rules
			if (distance <= MAX_WALK_DISTANCE)
				meansOfTransportation.add(new Integer(MeanOfTransportation.WALK));
				
			// taxi simple rules		
			if (distance >= MIN_TAXI_DISTANCE && distance <= MAX_TAXI_DISTANCE && distance > MAX_WALK_DISTANCE)
				meansOfTransportation.add(new Integer(MeanOfTransportation.TAXI));
			
			meansOfTransportation.add(new Integer(MeanOfTransportation.BUS));
			meansOfTransportation.add(new Integer(MeanOfTransportation.TRAIN));
			meansOfTransportation.add(new Integer(MeanOfTransportation.SHIP));
			meansOfTransportation.add(new Integer(MeanOfTransportation.AIRPLANE));
		}
					
		return meansOfTransportation;
	}
}
