/*
 * Created on 2005-08-30
 */
package scheduler.modulesinterfaces;

import scheduler.ConnectionsFinderTask;
import scheduler.TravelConstraints;
import scheduler.TravelPart;

/**
 * An interface that must be implemented by a class that will judge if to drop a specified changes or not.
 * @author Piotrek
 */
public interface ChangesJudger
{
	/**
	 * Checks whether it is wise to do not take the given change.
	 * @author Piotrek
	 * @param task A task describing the current connection.
	 * @param changeTask A task describing the change.
	 * @param travelPart A travel part that is being solved. It provides the destination.
	 * @param fRevert Specifies whether the planning is perfomred with the arrival date limit.
	 * @return A boolean value indicating whether to drop the given change.
	 */
	public boolean ShouldDropChange(ConnectionsFinderTask task, ConnectionsFinderTask changeTask, TravelPart travelPart, boolean fRevert);
	
	/**
	 * Sets the travel constraints providing the minimal duration of change.
	 * @author Piotrek
	 * @param constraints Travel constraints specifying the minimal duration of change.
	 */
	public void SetTravelConstraints(TravelConstraints constraints);
}
