/*
 * Created on 2005-08-29
 */
package scheduler.modulesinterfaces;

import java.util.ArrayList;
import scheduler.FeedbackStationsPair;
import scheduler.TravelConstraints;
import world.Location;
import world.MeanOfTransportation;

/**
 * An interface that must be implemented by a class providing the feedback travel parts (expert knowledge).
 * @author Piotrek
 */
public interface FeedbackTravelPartsProvider
{
	/**
	 * Gets a list of stations pairs to be taken into account when searching for the main parts of the travel division. 
	 * @author Piotrek
	 * @param origin An origin of the travel.
	 * @param destination A destination of the travel.
	 * @param meanOfTransportation A mean of transportation that is of interest.
	 * @see MeanOfTransportation#AIRPLANE
	 * @see MeanOfTransportation#BUS
	 * @see MeanOfTransportation#TRAIN
	 * @see MeanOfTransportation#SHIP
	 * @see FeedbackStationsPair
	 * @return A list of pairs of stations for connections of the given mean of transportation. Each element of the result list is of the FeedbackStationsPair type. If there are is no feedback, an empty list s returned.
	 */
	public ArrayList GetFeedbackTravelParts(Location origin, Location destination, int meanOfTransportation);
	
	/**
	 * Sets the travel constraints providing the forbidden means of transportation.
	 * @author Piotrek
	 * @param constraints Travel constraints specifying the forbidden means of transportation.
	 */
	public void SetTravelConstraints(TravelConstraints constraints);
}
