/*
 * Created on 2005-08-30
 */
package scheduler.modulesinterfaces;

import scheduler.ConnectionsFinderTask;

/**
 * An interface that must be implemented by a class that will judge the travel solutions.
 * @author Piotrek
 */
public interface TravelSolutionsJudger
{
	/**
	 * Judges the given travel solution.
	 * @author Piotrek
	 * @param travelSolution A travel solution to be judged.
	 * @return A similarity to the historical case. It is a value in range of [-1; 1], where -1 is 100% similarity to the rejected case and 1 is 100% similarity to the accepted case. Zero is returned if there is no historical data.
	 */
	public double JudgeTravelSolution(ConnectionsFinderTask travelSolution);
	
	/**
	 * Add the given travel solution to the history.
	 * @author Piotrek
	 * @param travelSolution A travel solution to be added.
	 * @param fAccepted Indicates whether this case has been accepted ot rejected by the user.
	 */
	public void AddTravelSolution(ConnectionsFinderTask travelSolution, boolean fAccepted);
}
