/*
 * Created on 2005-06-11
 */
package tests;

import java.util.ArrayList;
import java.util.Iterator;
import junit.framework.TestCase;
import crn.CRN;
import crn.Case;
import crn.IE;
import crn.IEComparator;

/**
 * Tests the CRN.
 * @author Piotrek
 */
public class CRNTest extends TestCase
{
	/**
	 * Tests adding of cases.
	 * @author Piotrek
	 */
	public void testAddCase()
	{
		IE ie1 = new IE("IE1", "value");
		IE ie2 = new IE("IE2", "value");
		
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		testCase1.AddIEReference(ie1);
		testCase2.AddIEReference(ie1);
		testCase2.AddIEReference(ie2);
		
		CRN crn = new CRN();
		
		boolean fThrown = false;
		
		try
		{
			crn.AddCase(null, 1.0f);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.AddCase() is not checking its parameters properly.");
		
		try
		{
			crn.AddCase(testCase1, 1.9f);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("CRN.AddCase() has thrown an exception for valid data.");
		}
		
		try
		{
			crn.AddCase(testCase2, 4.23f);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("CRN.AddCase() has thrown an exception for valid data.");
		}
		
		if (crn.GetIEWeight("IE1") != 1.9f)
			TestCase.fail("CRN.AddCase() is not working properly.");
		
		if (crn.GetIEWeight("IE2") != 4.23f)
			TestCase.fail("CRN.AddCase() is not working properly.");
		
		Iterator iterator = crn.GetCases();
	
		int counter = 0;
		
		while (iterator.hasNext())
		{
			iterator.next();
			
			++counter;
		}
		
		if (counter != 2)
			TestCase.fail("CRN.AddCase() is not working properly.");
	}

	/**
	 * Tests removing of the cases.
	 * @author Piotrek
	 */
	public void testRemoveCase()
	{
		IE ie1 = new IE("name1", "value1");
		IE ie2 = new IE("name3", "value2");
		IE ie3 = new IE("name1", "value3");
		IE ie4 = new IE("name3", "value4");
		
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		testCase1.AddIEReference(ie1);
		testCase1.AddIEReference(ie2);
		testCase2.AddIEReference(ie2);
		testCase2.AddIEReference(ie3);
		testCase2.AddIEReference(ie4);
		
		CRN crn = new CRN();
				
		boolean fThrown = false;
		
		try
		{
			crn.RemoveCase(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.RemoveCase() is not checking its parameter properly.");
		
		crn.AddCase(testCase1, 1.0f);
		crn.AddCase(testCase2, 1.0f);
		
		Case other = (Case)crn.GetCases().next();
		
		crn.RemoveCase(other);
		
		Iterator iterator = crn.GetCases();
		
		boolean fHas = false;
		
		int counter = 0;
		
		while (iterator.hasNext())
		{
			Case testCase = (Case)iterator.next();
			
			if (!testCase.IsEqual(other))
				fHas = true;
			
			++counter;
		}
		
		if (!fHas)
			TestCase.fail("CRN.RemoveCase() is not working properly.");
		
		if (counter != 1)
			TestCase.fail("CRN.RemoveCase() is not working properly.");
	}

	/**
	 * Tests searching for the similar cases.
	 * @author Piotrek
	 */
	public void testFindSimilarCases()
	{
		IE ie1 = new IE("name1", "value1");
		IE ie2 = new IE("name3", "value2");
		IE ie3 = new IE("name1", "value3");
		IE ie4 = new IE("name3", "value4");
		
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		testCase1.AddIEReference(ie1);
		testCase1.AddIEReference(ie2);
		testCase2.AddIEReference(ie2);
		testCase2.AddIEReference(ie3);
		testCase2.AddIEReference(ie4);
		
		CRN crn = new CRN();
		
		boolean fThrown = false;
		
		try
		{
			crn.FindSimilarCases(null, 1.0f);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.FindSimilarCases() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			crn.FindSimilarCases(testCase1, -0.1f);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.FindSimilarCases() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			crn.FindSimilarCases(testCase1, 1.1f);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.FindSimilarCases() is not checking its parameters properly.");
		
		crn.AddCase(testCase1, 1.0f);
		crn.AddCase(testCase2, 1.0f);
		
		Case testCase3 = new Case("c3", true, null);
		
		testCase3.AddIEReference(ie1);
		testCase3.AddIEReference(ie2);
		
		ArrayList similarOnes = null;
		
		try
		{
			similarOnes = crn.FindSimilarCases(testCase3, 0.9f);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("CRN.FindSimilarCases() has thrown an exception for valid parameters.");
		}
		
		if (similarOnes == null)
			TestCase.fail("CRN.FindSimilarCases() is not working properly.");
		
		if (similarOnes.size() != 1)
			TestCase.fail("CRN.FindSimilarCases() is not working properly.");
		
		similarOnes = crn.FindSimilarCases(testCase3, 1.0f);
		
		if (similarOnes == null)
			TestCase.fail("CRN.FindSimilarCases() is not working properly.");
		
		if (similarOnes.size() != 1)
			TestCase.fail("CRN.FindSimilarCases() is not working properly.");
				
		similarOnes = crn.FindSimilarCases(testCase3, 0.1f);
		
		if (similarOnes == null)
			TestCase.fail("CRN.FindSimilarCases() is not working properly.");
		
		if (similarOnes.size() != 2)
			TestCase.fail("CRN.FindSimilarCases() is not working properly.");
	}

	/**
	 * Test setting and getting of the IE weights.
	 * @author Piotrek
	 */
	public void testIEWeights()
	{
		CRN crn = new CRN();
		
		boolean fThrown = false;
		
		try
		{
			crn.GetIEWeight("test");
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.GetIEWeight() is not checking its parameter properly.");
		
		crn.SetIEWeight("test", 1.0f);
		crn.SetIEWeight("test2", 21.2f);
		
		if (crn.GetIEWeight("test") != 1.0f)
			TestCase.fail("CRN.GetIEWeight() is not working properly.");
		
		if (crn.GetIEWeight("test2") != 21.2f)
			TestCase.fail("CRN.GetIEWeight() is not working properly.");
	}

	/**
	 * Tests the parameter validation for adding of the specific comparator.
	 * @author Piotrek
	 */
	public void testAddSpecificComparator()
	{
		CRN crn = new CRN();
		
		boolean fThrown = false;
		
		try
		{
			crn.AddSpecificComparator(null, new IEComparator()
											{
												/* (non-Javadoc)
												 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
												 */
												public float CompareIEs(IE ie1, IE ie2)
												{													
													return 0;
												}
											}
									  );
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.AddSpecificComparator() is not checking its parameter properly.");
		
		fThrown = false;
		
		try
		{
			crn.AddSpecificComparator("test", null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("CRN.AddSpecificComparator() is not checking its parameter properly.");
		
		try
		{
			crn.AddSpecificComparator("aa", new IEComparator()
											{
												/* (non-Javadoc)
												 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
												 */
												public float CompareIEs(IE ie1, IE ie2)
												{													
													return 0;
												}
											}
									  );
		}
		
		catch (Throwable t)
		{
			TestCase.fail("CRN.AddSpecificComparator() has thrown an exception for valid parameters.");
		}
	}

	/**
	 * Tests only parameters validation for the removing of the specific comparator.
	 * @author Piotrek
	 */
	public void testRemoveSpecificComparator()
	{
		CRN crn = new CRN();
				
		try
		{
			crn.RemoveSpecificComparator(null);
		}
		
		catch (NullPointerException xcp)
		{
			return;
		}
		
		TestCase.fail("CRN.RemoveSpecificComparator() is not checking its parameter properly.");
	}

	/**
	 * Tests removal of all cases.
	 * @author Piotrek
	 */
	public void testRemoveAllCases()
	{
		IE ie1 = new IE("name1", "value1");
		IE ie2 = new IE("name3", "value2");
		IE ie3 = new IE("name1", "value3");
		IE ie4 = new IE("name3", "value4");
		
		Case testCase1 = new Case("c1", true, null);
		Case testCase2 = new Case("c2", true, null);
		
		testCase1.AddIEReference(ie1);
		testCase1.AddIEReference(ie2);
		testCase2.AddIEReference(ie2);
		testCase2.AddIEReference(ie3);
		testCase2.AddIEReference(ie4);
		
		CRN crn = new CRN();
		
		crn.AddCase(testCase1, 1.0f);
		crn.AddCase(testCase2, 1.0f);
		
		crn.RemoveAllCases();
		
		if (crn.GetCases().hasNext())
			TestCase.fail("CRN.RemoveAllCases() is not working properly.");
	}
}
