/*
 * Created on 2005-06-11
 */
package tests;

import java.util.ArrayList;
import java.util.Iterator;
import junit.framework.TestCase;
import crn.CRN;
import crn.Case;
import crn.IE;

/**
 * Tests the case.
 * @author Piotrek
 */
public class CaseTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testCase()
	{
		boolean fThrown = false;
		
		try
		{
			new Case(null, true, new CRN());
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Case.Case() is not checking its parameters properly.");		
				
		try
		{
			new Case("c1", true, new CRN());
		}
		
		catch (Throwable t)
		{
			TestCase.fail("Case.Case() has thrown an exception for the valid data.");
		}
		
		Case testCase = null;
		
		try
		{
			testCase = new Case("c1", true, null);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("Case.Case() has thrown an exception for the valid data.");
		}
				
		if (testCase.GetIEReferencesIterator().hasNext())
			TestCase.fail("There are some IEs just after creating a new case.");
	}

	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testEqualsObject()
	{
		CRN crn1 = new CRN();
		CRN crn2 = new CRN();
		
		Case testCase1 = new Case("a", true, crn1);
		Case testCase2 = new Case("b", true, crn1);
		Case testCase3 = new Case("a", false, crn1);
		Case testCase4 = new Case("a", true, crn2);
		Case testCase5 = new Case("a", true, crn1);
		Case testCase6 = new Case("c", false, crn1);
		
		if (testCase1.equals(testCase2))
			TestCase.fail("Case.equals() is not working properly.");
		
		if (testCase1.equals(testCase3))
			TestCase.fail("Case.equals() is not working properly.");
		
		if (!testCase1.equals(testCase4))
			TestCase.fail("Case.equals() is not working properly.");
		
		if (!testCase1.equals(testCase5))
			TestCase.fail("Case.equals() is not working properly.");
		
		if (testCase1.equals(testCase6))
			TestCase.fail("Case.equals() is not working properly.");
		
		if (testCase1.equals(new String("a")))
			TestCase.fail("Case.equals() is not working properly.");
		
		if (testCase1.equals(new Integer(4)))
			TestCase.fail("Case.equals() is not working properly.");
		
		boolean fThrown = false;
		
		try
		{
			testCase1.equals(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Case.equals() is not checking its parameter properly.");
	}

	/**
	 * Tests the IsEqual() method.
	 * @author Piotrek
	 */
	public void testIsEqual()
	{
		CRN crn1 = new CRN();
		CRN crn2 = new CRN();
		
		Case testCase1 = new Case("a", true, crn1);
		Case testCase2 = new Case("b", true, crn1);
		Case testCase3 = new Case("a", false, crn1);
		Case testCase4 = new Case("a", true, crn2);
		Case testCase5 = new Case("a", true, crn1);
		Case testCase6 = new Case("c", false, crn1);
		
		if (testCase1.IsEqual(testCase2))
			TestCase.fail("Case.IsEqual() is not working properly.");
		
		if (testCase1.IsEqual(testCase3))
			TestCase.fail("Case.IsEqual() is not working properly.");
		
		if (!testCase1.IsEqual(testCase4))
			TestCase.fail("Case.IsEqual() is not working properly.");
		
		if (!testCase1.IsEqual(testCase5))
			TestCase.fail("Case.IsEqual() is not working properly.");
		
		if (testCase1.IsEqual(testCase6))
			TestCase.fail("Case.IsEqual() is not working properly.");
				
		boolean fThrown = false;
		
		try
		{
			testCase1.IsEqual(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Case.IsEqual() is not checking its parameter properly.");
	}

	/**
	 * Tests adding of the IE references.
	 * @author Piotrek
	 */
	public void testAddIEReference()
	{
		IE ie1 = new IE("name", "value1");
		IE ie2 = new IE("name", "value2");
		
		Case testCase = new Case("1", true, new CRN());
		
		testCase.AddIEReference(ie1);
		testCase.AddIEReference(ie2);
		
		Iterator iterator = testCase.GetIEReferencesIterator();
		
		boolean fHasIE1 = false;
		boolean fHasIE2 = false;
		
		int counter = 0;
		
		while (iterator.hasNext())
		{
			IE ie = (IE)iterator.next();
			
			if (ie == ie1)
				fHasIE1 = true;
			
			if (ie == ie2)
				fHasIE2 = true;
			
			++counter;
		}
		
		if (!fHasIE1 || !fHasIE2)
			TestCase.fail("Case.AddIEReference() is not working properly.");
		
		if (counter != 2)
			TestCase.fail("Case.AddIEReference() is not working properly.");
	}

	/**
	 * Tests getting IEs with the given name.
	 * @author Piotrek
	 */
	public void testGetIEs()
	{
		IE ie1 = new IE("name1", "value1");
		IE ie2 = new IE("name1", "value2");
		IE ie3 = new IE("name2", "value3");
		IE ie4 = new IE("name3", "value4");
		
		Case testCase = new Case("1", true, new CRN());
		
		if (testCase.GetIEs("name1") == null)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		if (testCase.GetIEs("name2").size() > 0)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		testCase.AddIEReference(ie1);
		testCase.AddIEReference(ie2);
		testCase.AddIEReference(ie3);
		testCase.AddIEReference(ie4);
		
		ArrayList ies1 = testCase.GetIEs("name1");
		ArrayList ies2 = testCase.GetIEs("name2");
		ArrayList ies3 = testCase.GetIEs("name3");
		
		if (ies1.size() != 2)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		if (ies2.size() != 1)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		if (ies3.size() != 1)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		Iterator iterator = ies1.iterator();
		
		boolean fHasIE1 = false;
		boolean fHasIE2 = false;
		
		while (iterator.hasNext())
		{
			IE ie = (IE)iterator.next();
			
			if (ie == ie1)
				fHasIE1 = true;
			
			if (ie == ie2)
				fHasIE2 = true;
		}
		
		if (!fHasIE1 || !fHasIE2)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		if (ies2.get(0) != ie3)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		if (ies3.get(0) != ie4)
			TestCase.fail("Case.GetIEs() is not working properly.");
		
		if (testCase.GetIEs("noname").size() > 0)
			TestCase.fail("Case.GetIEs() is not working properly.");
	}
}
