/*
 * Created on 2005-05-14
 */
package tests;

import java.util.ArrayList;
import java.util.Calendar;
import junit.framework.TestCase;
import world.Airport;
import world.Connection;
import world.Location;
import world.MeanOfTransportation;
import world.TrainStation;

/**
 * Tests the Connection class.
 * @author Piotrek
 */
public class ConnectionTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testConnection()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		boolean fThrown = false;
		
		try
		{
			new Connection(null, MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not check its parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not check its parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", -2, stations, arrivals, departures, mask);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not check its parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, null, arrivals, departures, mask);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not check its parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, new ArrayList(), arrivals, departures, mask);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not check its parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, null, departures, mask);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not checkits parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, new ArrayList(), departures, mask);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not checkits parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, arrivals, null, mask);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not checkits parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, arrivals, new ArrayList(), mask);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not checkits parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, (char)0);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not checkits parameters thorougfully.");
		
		fThrown = false;
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, (char)255);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.Connection() does not checkits parameters thorougfully.");
		
		try
		{
			new Connection("1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("Connection.Connection() has thrown an exception for valid parameters: " + t.getMessage());
		}
	}
	
	/**
	 * Tests the method returning the index of the station on the list of stations of the connection.
	 * @author Piotrek
	 */
	public void testGetStationIndex()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		boolean fThrown = false;
		
		try
		{
			Connection connection = new Connection("test", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
			
			connection.GetStationIndex(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The Connection.GetStationMethod() has not thrown an exception when given a null reference.");
		
		fThrown = false;
		
		try
		{
			Connection connection = new Connection("test", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
			
			connection.GetStationIndex(new TrainStation(new Location(3, 4), "train station"));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The Connection.GetStationMethod() has not thrown an exception when given a station supporting some other mean of transportation.");
		
		Connection connection = new Connection("test", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		if (connection.GetStationIndex(new Airport(new Location(333, 444), "")) != -1)
			TestCase.fail("The Connection.GetStationMethod() has returned some other value then it ought to given a station supporting the right mean of transportation but which does not belong to the connection.");
		
		if (connection.GetStationIndex(new Airport(new Location(1, 2), "One")) != 0)
			TestCase.fail("The Connection.GetStationMethod() has returned some other value then it ought to given a station supporting the right mean of transportation and which does belong to the connection.");
		
		if (connection.GetStationIndex(new Airport(new Location(11, 12), "Two")) != 1)
			TestCase.fail("The Connection.GetStationMethod() has returned some other value then it ought to given a station supporting the right mean of transportation and which does belong to the connection.");
	}
	
	/**
	 * Tests getting of the arrival time for the stations.
	 * @author Piotrek
	 */
	public void testGetArrivalTime()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		boolean fThrown = false;
		
		try
		{
			connection.GetArrivalTime(-1);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.GetArrivalTime() is not checking indecies properly.");
		
		fThrown = false;
		
		try
		{
			connection.GetArrivalTime(3);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.GetArrivalTime() is not checking indecies properly.");
		
		if (connection.GetArrivalTime(0) != 60)
			TestCase.fail("Connection.GetArrivalTime() is not working properly.");
		
		if (connection.GetArrivalTime(1) != 120)
			TestCase.fail("Connection.GetArrivalTime() is not working properly.");
	}
	
	/**
	 * Tests getting of the departure time for the stations.
	 * @author Piotrek
	 */
	public void testGetDepartureTime()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		boolean fThrown = false;
		
		try
		{
			connection.GetDepartureTime(-1);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.GetDepartureTime() is not checking indecies properly.");
		
		fThrown = false;
		
		try
		{
			connection.GetDepartureTime(3);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.GetDepartureTime() is not checking indecies properly.");
		
		if (connection.GetDepartureTime(0) != 70)
			TestCase.fail("Connection.GetDepartureTime() is not working properly.");
		
		if (connection.GetDepartureTime(1) != 130)
			TestCase.fail("Connection.GetDepartureTime() is not working properly.");
	}
	
	/**
	 * Tests converting of java calendar day of week to the connection's day of week.
	 * @author Piotrek
	 */
	public void testCalendarToConnectionDayOfWeek()
	{
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.MONDAY) != Connection.MONDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.TUESDAY) != Connection.TUESDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.WEDNESDAY) != Connection.WEDNESDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.THURSDAY) != Connection.THURSDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.FRIDAY) != Connection.FRIDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.SATURDAY) != Connection.SATURDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		if (Connection.CalendarToConnectionDayOfWeek(Calendar.SUNDAY) != Connection.SUNDAY)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not working properly.");
		
		boolean fThrown = false;
		
		try
		{
			Connection.CalendarToConnectionDayOfWeek(99);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.CalendarToConnectionDayOfWeek() is not checking its parameter properly.");
	}
	
	/**
	 * Tests shifting days of week.
	 * @author Piotrek
	 */
	public void testShiftDayOfWeek()
	{
		boolean fThrown = false;
		
		try
		{
			Connection.ShiftDayOfWeek((char)9, 2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("Connection.ShiftDayOfWeek() is not checking its parameters properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 1) != Connection.TUESDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 2) != Connection.WEDNESDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 3) != Connection.THURSDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 4) != Connection.FRIDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 5) != Connection.SATURDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 6) != Connection.SUNDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 7) != Connection.MONDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, 8) != Connection.TUESDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.WEDNESDAY, 2) != Connection.FRIDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.MONDAY, -1) != Connection.SUNDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.TUESDAY, -7) != Connection.TUESDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
		
		if (Connection.ShiftDayOfWeek(Connection.FRIDAY, -15) != Connection.THURSDAY)
			TestCase.fail("Connection.ShiftDayOfWeek() is not working properly.");
	}
	
	/**
	 * Tests getting of the stations count.
	 * @author Piotrek
	 */
	public void testGetStationsCount()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		if (connection.GetStationsCount() != 2)
			TestCase.fail("Connection.GetStationsCount() is not working properly.");
	}
	
	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testEquals()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection1 = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		Connection connection2 = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		Connection connection3 = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, Connection.WEDNESDAY);
		
		if (!connection1.equals(connection2))
			TestCase.fail("Connection.equals() is not working properly.");
		
		if (connection1.equals(connection3))
			TestCase.fail("Connection.equals() is not working properly.");
		
		if (connection2.equals(connection3))
			TestCase.fail("Connection.equals() is not working properly.");
	}
	
	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testIsEqual()
	{
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection1 = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		Connection connection2 = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		Connection connection3 = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, Connection.WEDNESDAY);
		
		if (!connection1.IsEqual(connection2))
			TestCase.fail("Connection.equals() is not working properly.");
		
		if (connection1.IsEqual(connection3))
			TestCase.fail("Connection.equals() is not working properly.");
		
		if (connection2.IsEqual(connection3))
			TestCase.fail("Connection.equals() is not working properly.");
	}
}