/*
 * Created on 2005-06-11
 */
package tests;

import junit.framework.TestCase;
import crn.IE;
import crn.IEComparator;
import crn.IESpecificComparisonManager;

/**
 * Tests the IESpecificComparisonManager class.
 * @author Piotrek
 */
public class IESpecificComparisonManagerTest extends TestCase
{
	/**
	 * Tests adding and checking if there is a specific comparator for the given IE name.
	 * @author Piotrek
	 */
	public void testHasSpecificComparator()
	{
		IESpecificComparisonManager manager = new IESpecificComparisonManager();
		
		boolean fThrown = false;
		
		try
		{
			manager.HasSpecificComparator(null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.HasSpecificComparator() is not checking its parameter properly.");
		
		if (manager.HasSpecificComparator("test"))
			TestCase.fail("IESpecificComparisonManager.HasSpecificComparator() is not working properly.");
		
		manager.AddSpecificComparator("test", new IEComparator()
												{
													/* (non-Javadoc)
													 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
													 */
													public float CompareIEs(IE ie1, IE ie2)
													{
														return 0;
													}
												}
									  );
		
		if (!manager.HasSpecificComparator("test"))
			TestCase.fail("IESpecificComparisonManager.HasSpecificComparator() is not working properly.");
	}

	/**
	 * Tests removing of a specific comparator.
	 * @author Piotrek
	 */
	public void testRemoveSpecificComparator()
	{
		IESpecificComparisonManager manager = new IESpecificComparisonManager();
		
		manager.AddSpecificComparator("test", new IEComparator()
										{
											/* (non-Javadoc)
											 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
											 */
											public float CompareIEs(IE ie1, IE ie2)
											{
												return 0;
											}
										}
									);
		
		if (!manager.HasSpecificComparator("test"))
			TestCase.fail("IESpecificComparisonManager.HasSpecificComparator() is not working properly.");
		
		manager.RemoveSpecificComparator("test");
		
		if (manager.HasSpecificComparator("test"))
			TestCase.fail("IESpecificComparisonManager.RemoveSpecificComparator() is not working properly.");
	}

	/**
	 * Tests comparing of the IEs.
	 * @author Piotrek
	 */
	public void testCompareIEs()
	{
		IESpecificComparisonManager manager = new IESpecificComparisonManager();
		
		IE ie1 = new IE("ie1", "value");
		IE ie2 = new IE("ie2", "value");
		IE ie3 = new IE("ie2", "value2");
		
		boolean fThrown = false;
		
		try
		{
			manager.CompareIEs(null, ie2);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			manager.CompareIEs(ie1, null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() is not checking its parameters properly.");
		
		fThrown = false;
		
		try
		{
			manager.CompareIEs(ie1, ie2);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() has not detected that the given IEs have different names.");
		
		fThrown = false;
		
		try
		{
			manager.CompareIEs(ie3, ie2);
		}
		
		catch (RuntimeException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() has not detected that there is no scpecific comparator for the given IEs.");
		
		manager.AddSpecificComparator("ie2", new IEComparator()
										{
											/* (non-Javadoc)
											 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
											 */
											public float CompareIEs(IE ie1, IE ie2)
											{
												return 1.2f;
											}
										}
										);
		
		fThrown = false;
		
		try
		{
			manager.CompareIEs(ie3, ie2);
		}
		
		catch (RuntimeException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() has not detected that the scpecific comparator for the given IEs is not working properly.");
		
		manager.AddSpecificComparator("ie2", new IEComparator()
										{
											/* (non-Javadoc)
											 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
											 */
											public float CompareIEs(IE ie1, IE ie2)
											{
												return -0.2f;
											}
										}
										);

		fThrown = false;
		
		try
		{
			manager.CompareIEs(ie3, ie2);
		}
		
		catch (RuntimeException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() has not detected that the scpecific comparator for the given IEs is not working properly.");
		
		manager.AddSpecificComparator("ie2", new IEComparator()
										{
											/* (non-Javadoc)
											 * @see crn.IEComparator#CompareIEs(crn.IE, crn.IE)
											 */
											public float CompareIEs(IE ie1, IE ie2)
											{
												return 0.51f;
											}
										}
										);

		float similarity = 0;
		
		try
		{
			similarity = manager.CompareIEs(ie3, ie2);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("IESpecificComparisonManager.CompareIEs() has thrown exception when given valid parameters and there was a comparator.");
		}
		
		if (similarity != 0.51f)
			TestCase.fail("IESpecificComparisonManager.CompareIEs() has used invalid specific comparator.");
	}
}
