/*
 * Created on 2005-05-14
 */
package tests;

import gui.controls.WorldView;
import junit.framework.TestCase;
import world.Location;

/**
 * Tests the Location class.
 * @author Piotrek
 */
public class LocationTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testLocation()
	{
		Location location = new Location(1, 2);
		
		if (location.x != 1 || location.y != 2)
			TestCase.fail("A new instance of a location has unexpected coordinates assigned just after the creation.");
	}

	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testIsEqual()
	{
		Location location1 = new Location(1, 2);
		Location location2 = new Location(1, 2);
		
		Location location3 = new Location(12, 2);
		Location location4 = new Location(1, 22);
		Location location5 = new Location(13, 23);
		
		if (!location1.IsEqual(location2))
			TestCase.fail("The equality test has returned false for the identical locations.");
		
		if (location1.IsEqual(location3))
			TestCase.fail("The equality test has returned true for the different locations.");
		
		if (location1.IsEqual(location4))
			TestCase.fail("The equality test has returned true for the different locations.");
		
		if (location1.IsEqual(location5))
			TestCase.fail("The equality test has returned true for the different locations.");
	}

	/**
	 * Tests the distance solver.
	 * @author Piotrek
	 */
	public void testGetDistance()
	{
		Location location1 = new Location(0, 0);
		Location location2 = new Location(1, 0);
		Location location3 = new Location(0, 1);
		Location location4 = new Location(-1, 0);
		Location location5 = new Location(0, -1);
		Location location6 = new Location(4, 3);
		Location location7 = new Location(WorldView.width - 1, 1);
		
		if (Location.GetDistance(location1, location1) != 0)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location1, location2) != 1)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location1, location3) != 1)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location1, location4) != 1)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location1, location5) != 1)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location2, location4) != 2)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location4, location2) != 2)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location1, location6) != 5)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location6, location1) != 5)
			TestCase.fail("The distance solver is not working properly.");
		
		if (Location.GetDistance(location3, location7) != 1)
			TestCase.fail("The distance solver is not working properly."); 
	}

	/**
	 * Tests the `is in range' test.
	 * @author Piotrek
	 */
	public void testIsInRange()
	{
		Location location = new Location(4, 3);
		
		if (location.IsInRange(1, 1, 6, 2))
			TestCase.fail("Is in range test is not working properly.");
		
		if (!location.IsInRange(4, 3, 6, 6))
			TestCase.fail("Is in range test is not working properly.");
		
		if (!location.IsInRange(4, 3, 1, 2))
			TestCase.fail("Is in range test is not working properly.");
		
		if (!location.IsInRange(1, 1, 6, 7))
			TestCase.fail("Is in range test is not working properly.");
		
		if (!location.IsInRange(-1, 1, 6, 7))
			TestCase.fail("Is in range test is not working properly.");
		
		if (!location.IsInRange(1, 1, WorldView.width + 2, 7))
			TestCase.fail("Is in range test is not working properly.");
		
		if (!location.IsInRange(-10, 1, WorldView.width + 22, 7))
			TestCase.fail("Is in range test is not working properly.");
	}
}
