/*
 * Created on 2005-05-31
 */
package tests;

import java.util.ArrayList;
import java.util.Date;
import junit.framework.TestCase;
import scheduler.TravelPartConnectionChunk;
import world.Airport;
import world.Connection;
import world.Location;
import world.MeanOfTransportation;

/**
 * Tests the TravelPartConnectionChunk class.
 * @author Piotrek
 */
public class TravelPartConnectionChunkTest extends TestCase
{
	/**
	 * Tests if the use of this method is disabled.
	 * @author Piotrek
	 */
	public void testGetRevertedPart()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		ArrayList stations = new ArrayList(2);
		
		Airport airport1 = new Airport(new Location(1, 2), "One");
		Airport airport2 = new Airport(new Location(11, 12), "Two");
		
		stations.add(airport1);
		stations.add(airport2);
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		chunk.SetOriginStation(airport1);
		chunk.SetOriginStationIndex(0);
		
		chunk.SetDestinationStation(airport2);
		chunk.SetDestinationStationIndex(1);
		
		chunk.SetStartDate(new Date(111));
		chunk.SetEndDate(new Date(222));
		
		if (chunk.GetRevertedPart() == null)
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		TravelPartConnectionChunk revertedOne = (TravelPartConnectionChunk)chunk.GetRevertedPart();
		
		if (!revertedOne.GetRealConnection().IsEqual(connection))
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		if (revertedOne.GetOriginStation() != airport2)
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		if (revertedOne.GetDestinationStation() != airport1)
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		if (revertedOne.GetOriginStationIndex() != 1)
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		if (revertedOne.GetDestinationStationIndex() != 0)
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		if (!revertedOne.GetStartDate().equals(new Date(222)))
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
		
		if (!revertedOne.GetEndDate().equals(new Date(111)))
			TestCase.fail("TravelPartConnectionChunk.GetRevertedPart() is not working properly.");
	}

	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testTravelPartConnectionChunk()
	{
		boolean fThrown = false;
		
		try
		{
			new TravelPartConnectionChunk((char)99, new Location(1, 2), new Location(2, 3));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.TravelPartConnectionChunk() is not checking its parameter properly.");
		
		fThrown = false;
		
		try
		{
			new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, null, new Location(2, 3));
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.TravelPartConnectionChunk() is not checking its parameter properly.");
		
		fThrown = false;
		
		try
		{
			new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), null);
		}
		
		catch (NullPointerException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.TravelPartConnectionChunk() is not checking its parameter properly.");
		
		try
		{
			new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPartConnectionChunk.TravelPartConnectionChunk() has thrown an exception given valid parameters.");
		}
	}

	/**
	 * Test the getting of the index of the origin station.
	 * @author Piotrek
	 */
	public void testGetOriginStationIndex()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		chunk.SetOriginStationIndex(0);
		
		if (chunk.GetOriginStationIndex() != 0)
			TestCase.fail("TravelPartConnectionChunk.GetOriginStationIndex() is not working properly.");
		
		chunk.SetOriginStationIndex(1);
		
		if (chunk.GetOriginStationIndex() != 1)
			TestCase.fail("TravelPartConnectionChunk.GetOriginStationIndex() is not working properly.");
	}

	/**
	 * Test the setting of the index of the origin station.
	 * @author Piotrek
	 */
	public void testSetOriginStationIndex()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		boolean fThrown = false;
		
		try
		{
			chunk.SetOriginStationIndex(1);
		}
		
		catch (RuntimeException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.SetOriginStationIndex() has not detected that the connection is not set.");
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		fThrown = false;
		
		try
		{
			chunk.SetOriginStationIndex(-1);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.SetOriginStationIndex() is not checking its parameter properly.");
		
		fThrown = false;
		
		try
		{
			chunk.SetOriginStationIndex(2);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.SetOriginStationIndex() is not checking its parameter properly.");
		
		try
		{
			chunk.SetOriginStationIndex(1);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPartConnectionChunk.SetOriginStationIndex() has thrown an exception when it should not.");
		}
	}

	/**
	 * Test the getting of the index of the destination station.
	 * @author Piotrek
	 */
	public void testGetDestinationStationIndex()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		chunk.SetDestinationStationIndex(0);
		
		if (chunk.GetDestinationStationIndex() != 0)
			TestCase.fail("TravelPartConnectionChunk.GetDestinationStationIndex() is not working properly.");
		
		chunk.SetDestinationStationIndex(1);
		
		if (chunk.GetDestinationStationIndex() != 1)
			TestCase.fail("TravelPartConnectionChunk.GetDestinationStationIndex() is not working properly.");
	}

	/**
	 * Test the setting of the index of the destination station.
	 * @author Piotrek
	 */
	public void testSetDestinationStationIndex()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		boolean fThrown = false;
		
		try
		{
			chunk.SetDestinationStationIndex(1);
		}
		
		catch (RuntimeException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.SetDestinationStationIndex() has not detected that the connection is not set.");
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		fThrown = false;
		
		try
		{
			chunk.SetDestinationStationIndex(-1);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.SetDestinationStationIndex() is not checking its parameter properly.");
		
		fThrown = false;
		
		try
		{
			chunk.SetDestinationStationIndex(2);
		}
		
		catch (IndexOutOfBoundsException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelPartConnectionChunk.SetDestinationStationIndex() is not checking its parameter properly.");
		
		try
		{
			chunk.SetDestinationStationIndex(1);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPartConnectionChunk.SetDestinationStationIndex() has thrown an exception when it should not.");
		}
	}

	/**
	 * Test the getting of the connection this chunk is taken by.
	 * @author Piotrek
	 */
	public void testGetRealConnection()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		if (chunk.GetRealConnection() != null)
			TestCase.fail("TravelPartConnectionChunk.GetRealConnection() is not returning null at the begining.");
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		if (chunk.GetRealConnection() != connection)
			TestCase.fail("TravelPartConnectionChunk.GetRealConnection() is not working properly.");
	}

	/**
	 * Test the setting of the connection this chunk is taken by.
	 * @author Piotrek
	 */
	public void testSetRealConnection()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(new Airport(new Location(1, 2), "One"));
		stations.add(new Airport(new Location(11, 12), "Two"));
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		try
		{
			chunk.SetRealConnection(connection);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPartConnectionChunk.SetRealConnection() has thrown an exception given a valid parameter.");
		}
		
		if (chunk.GetRealConnection() != connection)
			TestCase.fail("TravelPartConnectionChunk.SetRealConnection() is not working properly.");
	}

	/**
	 * Test the getting of the start date.
	 * @author Piotrek
	 */
	public void testGetStartDate()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		if (chunk.GetStartDate() != null)
			TestCase.fail("TravelPartConnectionChunk.GetStartDate() is not returning null at the begining.");
		
		chunk.SetStartDate(new Date(111));
		
		if (!chunk.GetStartDate().equals(new Date(111)))
			TestCase.fail("TravelPartConnectionChunk.GetStartDate() is not working properly.");
		
		chunk.SetStartDate(null);
		
		if (chunk.GetStartDate() != null)
			TestCase.fail("TravelPartConnectionChunk.GetStartDate() is not working properly.");
	}

	/**
	 * Test the setting of the start date.
	 * @author Piotrek
	 */
	public void testSetStartDate()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
			
		try
		{
			chunk.SetStartDate(new Date(111));
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPartConnectionChunk.SetStartDate() has thrown an exception given a valid parameter.");
		}
	}

	/**
	 * Test the getting of the end date.
	 * @author Piotrek
	 */
	public void testGetEndDate()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		if (chunk.GetEndDate() != null)
			TestCase.fail("TravelPartConnectionChunk.GetEndDate() is not returning null at the begining.");
		
		chunk.SetEndDate(new Date(111));
		
		if (!chunk.GetEndDate().equals(new Date(111)))
			TestCase.fail("TravelPartConnectionChunk.GetEndDate() is not working properly.");
		
		chunk.SetEndDate(null);
		
		if (chunk.GetEndDate() != null)
			TestCase.fail("TravelPartConnectionChunk.GetEndDate() is not working properly.");
	}

	/**
	 * Test the setting of the end date.
	 * @author Piotrek
	 */
	public void testSetEndDate()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		try
		{
			chunk.SetEndDate(new Date(111));
		}
		
		catch (Throwable t)
		{
			TestCase.fail("TravelPartConnectionChunk.SetEndDate() has thrown an exception given a valid parameter.");
		}
	}

	/**
	 * Tests cloning.
	 * @author Piotrek
	 */
	public void testClone()
	{
		TravelPartConnectionChunk chunk = new TravelPartConnectionChunk(MeanOfTransportation.AIRPLANE, new Location(1, 2), new Location(2, 3));
		
		ArrayList stations = new ArrayList(2);
		
		Airport airport1 = new Airport(new Location(1, 2), "One");
		Airport airport2 = new Airport(new Location(11, 12), "Two");
		
		stations.add(airport1);
		stations.add(airport2);
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		char mask = Connection.MONDAY | Connection.FRIDAY;
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, mask);
		
		chunk.SetRealConnection(connection);
		
		chunk.SetOriginStation(airport1);
		chunk.SetOriginStationIndex(0);
		
		chunk.SetDestinationStation(airport2);
		chunk.SetDestinationStationIndex(1);
		
		chunk.SetStartDate(new Date(111));
		chunk.SetEndDate(new Date(222));
		
		TravelPartConnectionChunk clone = (TravelPartConnectionChunk)chunk.clone();
		
		if (clone.GetRealConnection() != connection)
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetOrigin() != chunk.GetOrigin())
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetDestination() != chunk.GetDestination())
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetMeanOfTransportation() != chunk.GetMeanOfTransportation())
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetOriginStation() != airport1)
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetOriginStationIndex() != 0)
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetDestinationStation() != airport2)
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (clone.GetDestinationStationIndex() != 1)
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (!clone.GetStartDate().equals(new Date(111)))
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
		
		if (!clone.GetEndDate().equals(new Date(222)))
			TestCase.fail("TravelPartConnectionChunk.clone() is not working properly.");
	}
}
