/*
 * Created on 2005-05-12
 */
package tests;

import junit.framework.TestCase;
import scheduler.TravelPart;
import scheduler.TravelPartsList;
import world.Location;
import world.MeanOfTransportation;
import world.Station;

/**
 * Tests the TravelPartsList class.
 * @author Piotrek
 */
public class TravelPartsListTest extends TestCase
{
	/**
	 * Tests the constructor.
	 * @author Piotrek
	 */
	public void testTravelPartsList()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
				
		if (travelPartsList.GetFirstTravelPart() != null)
			TestCase.fail("After the creation of the TravelParstList there must not be any parts in the list.");
	}

	/**
	 * Tests the adding of a single TravelPart to the list.
	 * @author Piotrek
	 */
	public void testAddTravelPart()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		TravelPart travelPart3 = new TravelPart(MeanOfTransportation.WALK, new Location(3, 2), new Location (3, 4));
		
		try
		{
			travelPartsList.AddTravelPart(travelPart1);
			
			if (travelPartsList.GetFirstTravelPart() != travelPart1 || travelPartsList.GetLastTravelPart() != travelPart1)
				TestCase.fail("A single TravelPart was not added correctly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("A single TravelPart was not added correctly.");
		}
		
		try
		{
			travelPartsList.AddTravelPart(travelPart2);
		
			if (travelPartsList.GetFirstTravelPart() != travelPart1 || travelPartsList.GetLastTravelPart() != travelPart2)
				TestCase.fail("A single TravelPart was not added correctly.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("A single TravelPart was not added correctly.");
		}
		
		boolean fThrown = false;
		
		try
		{
			travelPartsList.AddTravelPart(travelPart3);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("TravelParstList.AddTravelPart() has not detected a travel part starting not there, where ends the last travel part in the current list.");
	}

	/**
	 * Tests the adding of a list of parts.
	 * @author Piotrek
	 */
	public void testAddTravelParts()
	{
		TravelPartsList travelPartsList1 = new TravelPartsList();
		TravelPartsList travelPartsList2 = new TravelPartsList();
		TravelPartsList travelPartsList3 = new TravelPartsList();
		
		travelPartsList1.fContainsFeedback = false;
		travelPartsList2.fContainsFeedback = true;
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		TravelPart travelPart3 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(2, 2), new Location(3, 3));
		TravelPart travelPart4 = new TravelPart(MeanOfTransportation.TRAIN, new Location(3, 3), new Location(4, 4));
		TravelPart travelPart5 = new TravelPart(MeanOfTransportation.WALK, new Location(5, 4), new Location(6, 5));
				
		try
		{
			travelPartsList1.AddTravelPart(travelPart1);
			travelPartsList1.AddTravelPart(travelPart2);
		
			travelPartsList2.AddTravelPart(travelPart3);
			travelPartsList2.AddTravelPart(travelPart4);
			
			travelPartsList3.AddTravelPart(travelPart5);
		}
		
		catch (Throwable t)
		{
			TestCase.fail("Some single TravelParts were not added correctly.");
			
			return;
		}
		
		try
		{
			travelPartsList1.AddTravelParts(travelPartsList2);
			
			if (travelPartsList1.GetFirstTravelPart() != travelPart1 || travelPartsList1.GetLastTravelPart() != travelPart4)
				TestCase.fail("Addition of a valid parts list to the other parts list has failed to be done properly.");
			
			if (!travelPartsList1.fContainsFeedback)
				TestCase.fail("Addition of a travel parts list with the feedback hasn't caused this flag to be set.");
		}
		
		catch (Throwable t)
		{
			TestCase.fail("Addition of a valid parts list to the other parts list has thrown an exception.");
			
			return;
		}
		
		boolean fThrown = false;
		
		try
		{
			travelPartsList1.AddTravelParts(travelPartsList3);
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("The addition of a travel parts list to the other list has not thrown an exception despite the too long distance between the last and the first parts in the both lists.");
	}
	
	/**
	 * Tests the getting of changes count.
	 * @author Piotrek
	 */
	public void testGetChangesCount()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.WALK, new Location(20, 21), new Location(20, 20));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(20, 20), new Location(0, 0));
		TravelPart travelPart3 = new TravelPart(MeanOfTransportation.WALK, new Location(0, 0), new Location(1, 0));
		TravelPart travelPart4 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 0), new Location(2, 0));
		TravelPart travelPart5 = new TravelPart(MeanOfTransportation.WALK, new Location(2, 0), new Location(3, 0));
		TravelPart travelPart6 = new TravelPart(MeanOfTransportation.BUS, new Location(3, 0), new Location(5, 0));
		TravelPart travelPart7 = new TravelPart(MeanOfTransportation.TRAIN, new Location(5, 0), new Location(6, 0));
		
		travelPartsList.AddTravelPart(travelPart1);
		
		if (travelPartsList.GetChangesCount() != 0)
			TestCase.fail("The getting of changes count is not working properly.");
		
		travelPartsList.AddTravelPart(travelPart2);
		
		if (travelPartsList.GetChangesCount() != 0)
			TestCase.fail("The getting of changes count is not working properly.");
		
		travelPartsList.AddTravelPart(travelPart3);
		
		if (travelPartsList.GetChangesCount() != 0)
			TestCase.fail("The getting of changes count is not working properly.");
		
		travelPartsList.AddTravelPart(travelPart4);
		
		if (travelPartsList.GetChangesCount() != 1)
			TestCase.fail("The getting of changes count is not working properly.");
		
		travelPartsList.AddTravelPart(travelPart5);
		
		if (travelPartsList.GetChangesCount() != 1)
			TestCase.fail("The getting of changes count is not working properly.");
		
		travelPartsList.AddTravelPart(travelPart6);
		
		if (travelPartsList.GetChangesCount() != 2)
			TestCase.fail("The getting of changes count is not working properly.");
		
		travelPartsList.AddTravelPart(travelPart7);
		
		if (travelPartsList.GetChangesCount() != 3)
			TestCase.fail("The getting of changes count is not working properly.");
	}

	/**
	 * Tests the getting of the first part from the list.
	 * @author Piotrek
	 */
	public void testGetFirstTravelPart()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		
		travelPartsList.AddTravelPart(travelPart1);
		travelPartsList.AddTravelPart(travelPart2);
		
		if (travelPartsList.GetFirstTravelPart() != travelPart1)
			TestCase.fail("The returned element is not the first in the list as expected.");
	}

	/**
	 * Tests the getting of the last part from the list.
	 * @author Piotrek
	 */
	public void testGetLastTravelPart()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		
		travelPartsList.AddTravelPart(travelPart1);
		travelPartsList.AddTravelPart(travelPart2);
		
		if (travelPartsList.GetLastTravelPart() != travelPart2)
			TestCase.fail("The returned element is not the last in the list as expected.");
	}

	/**
	 * Tests the getting of the count of travel parts. 
	 * @author Piotrek
	 */
	public void testGetTravelPartsCount()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
		
		if (travelPartsList.GetTravelPartsCount() != 0)
			TestCase.fail("The list ought to be empty.");
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		
		travelPartsList.AddTravelPart(travelPart1);
		
		if (travelPartsList.GetTravelPartsCount() != 1)
			TestCase.fail("The list ought to contain one element.");
		
		travelPartsList.AddTravelPart(travelPart2);
		
		if (travelPartsList.GetTravelPartsCount() != 2)
			TestCase.fail("The list ought to contain two element.");
	}

	/**
	 * Tests the getting of the total length of all the parts in the list. 
	 * @author Piotrek
	 */
	public void testGetLength()
	{
		TravelPartsList travelPartsList = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 0));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 0), new Location(2, 0));
		TravelPart travelPart3 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(2, 0), new Location(3, 0));
		TravelPart travelPart4 = new TravelPart(MeanOfTransportation.TRAIN, new Location(3, 0), new Location(5, 0));
		
		travelPartsList.AddTravelPart(travelPart1);
		
		if (travelPartsList.GetLength() != 1)
			TestCase.fail("The total length of the travel parts ought to be 1.");
		
		travelPartsList.AddTravelPart(travelPart2);
		
		if (travelPartsList.GetLength() != 2)
			TestCase.fail("The total length of the travel parts ought to be 2.");
		
		travelPartsList.AddTravelPart(travelPart3);
		
		if (travelPartsList.GetLength() != 3)
			TestCase.fail("The total length of the travel parts ought to be 3.");
		
		travelPartsList.AddTravelPart(travelPart4);
		
		if (travelPartsList.GetLength() != 5)
			TestCase.fail("The total length of the travel parts ought to be 5.");
	}

	/**
	 * Tests the equality test.
	 * @author Piotrek
	 */
	public void testIsEqual()
	{
		TravelPartsList travelPartsList1 = new TravelPartsList();
		TravelPartsList travelPartsList2 = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		TravelPart travelPart3 = new TravelPart(MeanOfTransportation.SHIP, new Location(2, 2), new Location(3, 3));
		TravelPart travelPart4 = new TravelPart(MeanOfTransportation.BUS, new Location(3, 3), new Location(4, 3));
		
		travelPartsList1.AddTravelPart(travelPart1);
		travelPartsList1.AddTravelPart(travelPart2);
		travelPartsList1.AddTravelPart(travelPart3);
		
		travelPartsList2.AddTravelPart(travelPart1);
		travelPartsList2.AddTravelPart(travelPart2);
		travelPartsList2.AddTravelPart(travelPart3);
		
		if (!travelPartsList1.IsEqual(travelPartsList2))
			TestCase.fail("The two travel parts lists ought to be considered equal.");
		
		travelPartsList2.AddTravelPart(travelPart4);
		
		if (travelPartsList1.IsEqual(travelPartsList2))
			TestCase.fail("The two travel parts lists should not be considered equal.");
		
		travelPartsList2 = new TravelPartsList();
		
		travelPart3 = new TravelPart(MeanOfTransportation.SHIP, new Location(2, 2), new Location(3, 3));
		travelPart3.SetOriginStation(new Station(new Location(1, 1), "test name", MeanOfTransportation.AIRPLANE));
		
		travelPartsList2.AddTravelPart(travelPart1);
		travelPartsList2.AddTravelPart(travelPart2);
		travelPartsList2.AddTravelPart(travelPart3);
		
		if (travelPartsList1.IsEqual(travelPartsList2))
			TestCase.fail("The two travel parts lists should not be considered equal.");
	}
	
	/**
	 * Tests getting of the reverted list.
	 * @author Piotrek
	 */
	public void testGetRevertList()
	{
		TravelPartsList travelPartsList1 = new TravelPartsList();
		TravelPartsList travelPartsList2 = new TravelPartsList();
		
		TravelPart travelPart1 = new TravelPart(MeanOfTransportation.AIRPLANE, new Location(0, 0), new Location(1, 1));
		TravelPart travelPart2 = new TravelPart(MeanOfTransportation.TRAIN, new Location(1, 1), new Location(2, 2));
		TravelPart travelPart3 = new TravelPart(MeanOfTransportation.SHIP, new Location(2, 2), new Location(3, 3));
		
		travelPartsList1.AddTravelPart(travelPart1);
		travelPartsList1.AddTravelPart(travelPart2);
		travelPartsList1.AddTravelPart(travelPart3);
		
		travelPartsList2.AddTravelPart(travelPart3.GetRevertedPart());
		travelPartsList2.AddTravelPart(travelPart2.GetRevertedPart());
		travelPartsList2.AddTravelPart(travelPart1.GetRevertedPart());
		
		if (!travelPartsList1.GetRevertedList().IsEqual(travelPartsList2))
			TestCase.fail("The reverting of the travel parts list does not work properly.");
	}
	
	/**
	 * Tests the initial value of the fContainsFeedback flag.
	 * @author Piotrek
	 */
	public void testInitialValueOfFContainsFeedback()
	{
		TravelPartsList list = new TravelPartsList();
		
		if (list.fContainsFeedback)
			TestCase.fail("Travel parts list has the fConstainsFeedback set by default.");
	}
}
