/*
 * Created on 2005-05-14
 */
package tests;

import java.util.ArrayList;
import junit.framework.TestCase;
import world.Airport;
import world.BusStop;
import world.City;
import world.Connection;
import world.Harbour;
import world.Location;
import world.MeanOfTransportation;
import world.TrainStation;
import world.World;

/**
 * Tests the World class.
 * @author Piotrek
 */
public class WorldTest extends TestCase
{
	/**
	 * Tests the getting of the World instance.
	 * @author Piotrek
	 */
	public void testGetInstance()
	{
		World firstCallWorld = World.GetInstance();
		
		if (firstCallWorld == null)
			TestCase.fail("Cannot get the instance of the world.");
		
		if (World.GetInstance() != firstCallWorld)
			TestCase.fail("Two calls to the World.GetInstance() has returned a different references without a call to the World.Clear() between them.");
	}

	/**
	 * Tests the Clear() method.
	 * @author Piotrek
	 */
	public void testClear()
	{
		World world = World.GetInstance();
		
		world.AddAirport(new Airport(new Location(1, 1), "A1"));
		world.AddAirport(new Airport(new Location(100, 100), "A2"));
		world.AddBusStop(new BusStop(new Location(2, 1), "B1"));
		world.AddCity(new City(new Location(0, 1), "C1"));
		world.AddHarbour(new Harbour(new Location(3, 1), "H1"));
		world.AddTrainStation(new TrainStation(new Location(4, 1), "T1"));
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(world.GetAirports().next());
		stations.add(world.GetAirports().next());		
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		world.AddConnection(new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, Connection.SUNDAY));
		
		World.Clear();
		
		world = World.GetInstance();
		
		if (world.GetAirports().hasNext())
			TestCase.fail("World.Clear failed to clear the airports.");
		
		if (world.GetBusStops().hasNext())
			TestCase.fail("World.Clear failed to clear the bus stops.");
		
		if (world.GetTrainStations().hasNext())
			TestCase.fail("World.Clear failed to clear the train stations.");
		
		if (world.GetHarbours().hasNext())
			TestCase.fail("World.Clear failed to clear the harbours.");
		
		if (world.GetCities().hasNext())
			TestCase.fail("World.Clear failed to clear the cities.");
		
		if (world.GetConnections().hasNext())
			TestCase.fail("World.Clear failed to clear the connections.");
	}

	/**
	 * Tests the adding of cities.
	 * @author Piotrek
	 */
	public void testAddCity()
	{
		World.Clear();
		
		City city = new City(new Location(0, 0), "City"); 
		
		World.GetInstance().AddCity(city);
		
		if (!World.GetInstance().GetCities().hasNext() || World.GetInstance().GetCities().next() != city)
			TestCase.fail("Adding of cities to the world is not working properly.");
	}

	/**
	 * Tests the removing of cities from the world.
	 * @author Piotrek
	 */
	public void testRemoveCity()
	{
		World.Clear();
		
		City city = new City(new Location(0, 0), "City"); 
		
		World.GetInstance().AddCity(city);
		
		World.GetInstance().RemoveCity(city);
		
		if (World.GetInstance().GetCities().hasNext())
			TestCase.fail("Removing of cities from the world is not working properly.");
	}

	/**
	 * Tests the addition of airports to the world.
	 * @author Piotrek
	 */
	public void testAddAirport()
	{
		World.Clear();
		
		Airport airport = new Airport(new Location(0, 0), "Airport"); 
		
		World.GetInstance().AddAirport(airport);
		
		if (!World.GetInstance().GetAirports().hasNext() || World.GetInstance().GetAirports().next() != airport)
			TestCase.fail("Adding of airports to the world is not working properly.");
	}

	/**
	 * Tests the removing of airports from the world.
	 * @author Piotrek
	 */
	public void testRemoveAirport()
	{
		World.Clear();
		
		Airport airport = new Airport(new Location(0, 0), "Airport"); 
		
		World.GetInstance().AddAirport(airport);
		
		World.GetInstance().RemoveAirport(airport);
		
		if (World.GetInstance().GetAirports().hasNext())
			TestCase.fail("Removing of airports from the world is not working properly.");
	}

	/**
	 * Tests the adding of the train stations to the world.
	 * @author Piotrek
	 */
	public void testAddTrainStation()
	{
		World.Clear();
		
		TrainStation trainStation = new TrainStation(new Location(0, 0), "TrainStation"); 
		
		World.GetInstance().AddTrainStation(trainStation);
		
		if (!World.GetInstance().GetTrainStations().hasNext() || World.GetInstance().GetTrainStations().next() != trainStation)
			TestCase.fail("Adding of train stations to the world is not working properly.");
	}

	/**
	 * Tests the removing of the train stations from the world.
	 * @author Piotrek
	 */
	public void testRemoveTrainStation()
	{
		World.Clear();
		
		TrainStation trainStation = new TrainStation(new Location(0, 0), "TrainStation"); 
		
		World.GetInstance().AddTrainStation(trainStation);
		
		World.GetInstance().RemoveTrainStation(trainStation);		
		
		if (World.GetInstance().GetTrainStations().hasNext())
			TestCase.fail("Removing of train stations from the world is not working properly.");
	}

	/**
	 * Tests adding of bus stops to the world.
	 * @author Piotrek
	 */
	public void testAddBusStop()
	{
		World.Clear();
		
		BusStop busStop = new BusStop(new Location(0, 0), "BusStop"); 
		
		World.GetInstance().AddBusStop(busStop);
		
		if (!World.GetInstance().GetBusStops().hasNext() || World.GetInstance().GetBusStops().next() != busStop)
			TestCase.fail("Adding of bus stops to the world is not working properly.");
	}

	/**
	 * Tests the removing of bus stops from the world.
	 * @author Piotrek
	 */
	public void testRemoveBusStop()
	{
		World.Clear();
		
		BusStop busStop = new BusStop(new Location(0, 0), "BusStop"); 
		
		World.GetInstance().AddBusStop(busStop);
		
		World.GetInstance().RemoveBusStop(busStop);
		
		if (World.GetInstance().GetBusStops().hasNext())
			TestCase.fail("Removing of bus stops from the world is not working properly.");
	}

	/**
	 * Tests the adding of harbours to the world.
	 * @author Piotrek
	 */
	public void testAddHarbour()
	{
		World.Clear();
		
		Harbour habrour = new Harbour(new Location(0, 0), "Harbour"); 
		
		World.GetInstance().AddHarbour(habrour);
		
		if (!World.GetInstance().GetHarbours().hasNext() || World.GetInstance().GetHarbours().next() != habrour)
			TestCase.fail("Adding of harbours to the world is not working properly.");
	}

	/**
	 * Tests the removing of the harbours from the world.
	 * @author Piotrek
	 */
	public void testRemoveHarbour()
	{
		World.Clear();
		
		Harbour harbour = new Harbour(new Location(0, 0), "Harbour"); 
		
		World.GetInstance().AddHarbour(harbour);
		
		World.GetInstance().RemoveHarbour(harbour);
		
		if (World.GetInstance().GetHarbours().hasNext())
			TestCase.fail("Removing of from the world is not working properly.");
	}

	/**
	 * Tests adding of connections to the world.
	 * @author Piotrek
	 */
	public void testAddConnection()
	{
		World.Clear();
		
		World world = World.GetInstance();
		
		world.AddAirport(new Airport(new Location(1, 1), "A1"));
		
		ArrayList airports = new ArrayList(2);
		
		airports.add(world.GetAirports().next());
		airports.add(world.GetAirports().next());		
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		Connection connection = new Connection("Flight #2", MeanOfTransportation.AIRPLANE, airports, arrivals, departures, Connection.SUNDAY);
		
		world.AddConnection(connection);
		
		if (!World.GetInstance().GetConnections().hasNext() || World.GetInstance().GetConnections().next() != connection)
			TestCase.fail("Adding of connections to the world is not working properly.");
				
		boolean fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Bad connection name", MeanOfTransportation.AIRPLANE, airports, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new air connection.");
		
		// for other means of transportation we will use airports, but this doesn't matter
		fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Bad connection name", MeanOfTransportation.BUS, airports, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new bus connection.");
		
		fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Bad connection name", MeanOfTransportation.TRAIN, airports, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new railway connection.");
		
		fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Bad connection name", MeanOfTransportation.SHIP, airports, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new sea connection.");
		
		// connection numbers tests
		
		fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Flight #1", MeanOfTransportation.AIRPLANE, airports, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new air connection.");
		
		world.AddBusStop(new BusStop(new Location(1, 1), "B1"));
		
		ArrayList busStops = new ArrayList(2);
		
		busStops.add(world.GetBusStops().next());
		busStops.add(world.GetBusStops().next());
				
		fThrown = false;
		
		World.GetInstance().AddConnection(new Connection("Bus #2", MeanOfTransportation.BUS, busStops, arrivals, departures, Connection.SUNDAY));
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Bus #1", MeanOfTransportation.BUS, busStops, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new bus connection.");
		
		world.AddTrainStation(new TrainStation(new Location(1, 1), "T1"));
		
		ArrayList trainStations = new ArrayList(2);
		
		trainStations.add(world.GetTrainStations().next());
		trainStations.add(world.GetTrainStations().next());
		
		fThrown = false;
		
		World.GetInstance().AddConnection(new Connection("Train #2", MeanOfTransportation.TRAIN, trainStations, arrivals, departures, Connection.SUNDAY));
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Train #1", MeanOfTransportation.TRAIN, trainStations, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new railway connection.");
		
		world.AddHarbour(new Harbour(new Location(1, 1), "H1"));
		
		ArrayList harbours = new ArrayList(2);
		
		harbours.add(world.GetHarbours().next());
		harbours.add(world.GetHarbours().next());
		
		fThrown = false;
		
		World.GetInstance().AddConnection(new Connection("Cruise #2", MeanOfTransportation.SHIP, harbours, arrivals, departures, Connection.SUNDAY));
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Cruise #1", MeanOfTransportation.SHIP, harbours, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly the name of the new sea connection.");
		
		// tests of stations out of world
						
		fThrown = false;
		
		airports.add(0, new Airport(new Location(2, 2), "A2"));
		airports.remove(2);
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Flight #3", MeanOfTransportation.AIRPLANE, airports, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly if the world contains all the stations belonging to the new connection.");
		
		world.AddBusStop(new BusStop(new Location(1, 1), "B1"));
			
		busStops.add(0, new BusStop(new Location(2, 2), "B2"));
		busStops.remove(2);
				
		fThrown = false;
				
		try
		{
			World.GetInstance().AddConnection(new Connection("Bus #3", MeanOfTransportation.BUS, busStops, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly if the world contains all the stations belonging to the new connection.");
		
		trainStations.add(0, new TrainStation(new Location(2, 2), "T2"));
		trainStations.remove(2);
		
		fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Train #3", MeanOfTransportation.TRAIN, trainStations, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly if the world contains all the stations belonging to the new connection.");
		
		harbours.add(0, new Harbour(new Location(2, 2), "H2"));
		harbours.remove(2);
		
		fThrown = false;
		
		try
		{
			World.GetInstance().AddConnection(new Connection("Cruise #3", MeanOfTransportation.SHIP, harbours, arrivals, departures, Connection.SUNDAY));
		}
		
		catch (IllegalArgumentException xcp)
		{
			fThrown = true;
		}
		
		if (!fThrown)
			TestCase.fail("World.AddConnection() does not check properly if the world contains all the stations belonging to the new connection.");
	}

	/**
	 * Tests the removing of connections from the world.
	 * @author Piotrek
	 */
	public void testRemoveConnection()
	{
		World.Clear();
		
		World world = World.GetInstance();
		
		world.AddAirport(new Airport(new Location(1, 1), "A1"));
		world.AddAirport(new Airport(new Location(100, 100), "A2"));
		
		ArrayList stations = new ArrayList(2);
		
		stations.add(world.GetAirports().next());
		stations.add(world.GetAirports().next());		
		
		ArrayList arrivals = new ArrayList(2);
		
		arrivals.add(new Integer(60));
		arrivals.add(new Integer(120));
		
		ArrayList departures = new ArrayList(2);
		
		departures.add(new Integer(70));
		departures.add(new Integer(130));
		
		Connection connection = new Connection("Flight #1", MeanOfTransportation.AIRPLANE, stations, arrivals, departures, Connection.SUNDAY);
		
		world.AddConnection(connection);
		
		world.RemoveConnection(connection);
		
		if (World.GetInstance().GetConnections().hasNext())
			TestCase.fail("Removing of connections from the world is not working properly.");
	}

	/**
	 * Tests the getting of names for the connections.
	 * @author Piotrek
	 */
	public void testGetNextConnectionName()
	{
		World.Clear();
		
		World world = World.GetInstance();
		
		if (world.GetNextConnectionName(MeanOfTransportation.AIRPLANE).compareTo("Flight #1") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the air connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.AIRPLANE).compareTo("Flight #2") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the air connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.BUS).compareTo("Bus #1") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the bus connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.BUS).compareTo("Bus #2") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the bus connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.TRAIN).compareTo("Train #1") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the railway connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.TRAIN).compareTo("Train #2") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the railway connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.SHIP).compareTo("Cruise #1") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the sea connection.");
		
		if (world.GetNextConnectionName(MeanOfTransportation.SHIP).compareTo("Cruise #2") != 0)
			TestCase.fail("The World.GetNextConnectionName() has returned an incorrect name for the sea connection.");
	}
}
