/*
 * Created on 2005-04-17
 */
package world;


/**
 * A station class. All stations derive from it.
 * @author Piotrek
 */
public class Station extends WorldObject
{
	/**
	 * Constructor.
	 * @author Piotrek
	 * @param location A location of the new station.
	 * @param name A name of this station.
	 * @param meanOfTransportation A mean of transportation this station support.
	 */
	public Station(Location location, String name, int meanOfTransportation)
	{
		// the base class checks for the null values
		super(location, name);
		
		if (!MeanOfTransportation.IsValidMeanOfTransportation(meanOfTransportation))
			throw new IllegalArgumentException("Unecpected value of the mean of transportation in Station.Station().");
		
		this.meanOfTransportation = meanOfTransportation;
	}
	
	/**
	 * A mean of transportation supported by this station.
	 * @author Piotrek
	 */
	private int meanOfTransportation = -1;
	
	/**
	 * Gets a mean of transportation this station supports.
	 * @author Piotrek
	 * @return A mean of transportation this station supports.
	 */
	public int GetMeanOfTransportation()
	{
		return meanOfTransportation;
	}
	
	/**
	 * Gets a boolean value indicating whether this and the other one stations are equal.
	 * @author Piotrek
	 * @param other The other station to compare this one to.
	 * @return A boolean value indicating whether this and the other one stations are equal.
	 */
	public boolean IsEqual(Station other)
	{		
		return (other != null && GetLocation().IsEqual(other.GetLocation()) && meanOfTransportation == other.GetMeanOfTransportation() && name.compareTo(other.name) == 0);
	}
	
	/**
	 * @author Piotrek
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object other)
	{
		if (!(other instanceof Station))
			return false;
		
		return IsEqual((Station)other);
	}
}
